@@ -1,12 +1,13 @@
 import gtk
 
-import frontend
 import threading
 from frontend_implementation.gtk_queue import queue
-import gettext
-import locale
+import gtcache
+import config
+import prefs
 import gtk.glade
 import resource
+import platformutils
 
 ###############################################################################
 #### Application object                                                    ####
@@ -15,25 +16,18 @@ import resource
 class Application:
 
     def __init__(self):
-	#print "Application init"
+        #print "Application init"
         pass
-
     def Run(self):
-        locale.setlocale(locale.LC_ALL, '')
-        gettext.bindtextdomain("democracyplayer", resource.path("../../locale"))
-        gettext.textdomain("democracyplayer")
-        gettext.bind_textdomain_codeset("democracyplayer","UTF-8")
-        gtk.glade.bindtextdomain("democracyplayer", resource.path("../../locale"))
+        gtk.glade.bindtextdomain("democracyplayer", config.get(prefs.GETTEXT_PATHNAME))
         gtk.glade.textdomain("democracyplayer")
 
         queue.main_thread = threading.currentThread()
+        platformutils.setMainThread()
         gtk.threads_init()
-	self.onStartup()
+        self.onStartup()
         gtk.main()
-	self.onShutdown()
-
-    def getBackendDelegate(self):
-        return frontend.UIBackendDelegate()
+        self.onShutdown()
 
     def onStartup(self):
         # For overriding
