@@ -4,6 +4,8 @@ import copy
 import sys
 import string
 import subprocess
+import zipfile as zip
+from glob import glob
 
 ###############################################################################
 ## Paths and configuration                                                   ##
@@ -21,7 +23,7 @@ NSIS_PATH = 'C:\\Program Files\\NSIS\\makensis.exe'
 # the Binary Kit, ignore this setting and change all of the settings
 # below.
 defaultBinaryKitRoot = os.path.join(os.path.dirname(sys.argv[0]), \
-				    '..', '..', '..', 'dtv-binary-kit')
+                                    '..', '..', '..', 'dtv-binary-kit')
 BINARY_KIT_ROOT = defaultBinaryKitRoot
 
 # Set BOOST_LIB, BOOST_INCLUDE_PATH, and BOOST_RUNTIMES as appropriate
@@ -149,27 +151,22 @@ ext_modules = [
     # Pyrex sources.
     #Extension("vlc", [os.path.join(root, 'platform',platform, 'vlc.pyx')],libraries=["simplevlc"]),
     Extension("database", [os.path.join(root, 'portable', 'database.pyx')]),
+    Extension("sorts", [os.path.join(root, 'portable', 'sorts.pyx')]),
     #Extension("template", [os.path.join(root, 'portable', 'template.pyx')]),
 ]
 
 import template_compiler
 
+def fillTemplate(templatepath, outpath, **vars):
+    s = open(templatepath, 'rt').read()
+    s = string.Template(s).safe_substitute(**vars)
+    f = open(outpath, "wt")
+    f.write(s)
+    f.close()
+
+
 ###############################################################################
 
-# WHEN LAUNCHING DIRECTLY ('test mode'):
-#
-# 1) Build any extensions; add results to sys.path
-# 2) Add all application Python directories to sys.path
-# 3) Find 'resources'
-# 4) Build a complete xulrunner image in a temporary directory
-# 5) Find the application template, including application.ini
-# 6) Compile all IDL files in 'idl'. [HACK: save results into the original
-#    copy of the application tree..]
-# 7) Adjust PATH and PYTHONPATH. Set RUNXUL_RESOURCES to point at 'resources'.
-#    to tell child process where to find them.
-# 8) Shell out to the xulrunner binary in the built xulrunner image, passing
-#    the path to application.ini.
-#
 # WHEN BUILDING A REDISTRIBUTABLE IMAGE:
 #
 # 1) Build any extensions; add results to sys.path
@@ -193,282 +190,12 @@ import template_compiler
 # if present. resource.resourceRoot() has already been modified in what
 # should be the appropriate way.
 
-class Common:
-    def __init__(self):
-        self.templateVars = None
-
-    def buildDownloadDaemon(self, baseDir):
-        print "building download daemon"
-        os.system("%s setup_daemon.py py2exe --dist-dir daemon --bundle-files 1" % PYTHON_BINARY)
-        shutil.copy2(os.path.join("daemon","Democracy_Downloader.exe"), baseDir)
-
-    # NEEDS: if you look at the usage of this function, we're dropping
-    # the plugin into the xulrunner plugin directory, rather than the
-    # app bundle plugin directory, which is the way you're "supposed"
-    # to do it so your app is cleanly separated from xulrunner.
-    def copyVLCPluginFiles(self, baseDir, xulrunnerDir):
-        destDir = os.path.join(xulrunnerDir, 'plugins')
-	if not os.access(destDir, os.F_OK):
-	    os.mkdir(destDir)
-
-        pluginFiles = ['npvlc.dll', 'vlcintf.xpt']
-        for f in pluginFiles:
-            shutil.copy2(os.path.join(VLC_MOZ_PLUGIN_DIR, f), destDir)
-
-        vlcPluginDest = os.path.join(baseDir, "vlc-plugins")
-	if not os.access(vlcPluginDest, os.F_OK):
-	    os.mkdir(vlcPluginDest)
-
-        vlcPlugins = os.listdir(VLC_PLUGINS_DIR)
-        for f in vlcPlugins:
-            if f[0] != '.':
-                shutil.copy2(os.path.join(VLC_PLUGINS_DIR, f), vlcPluginDest)
-
-    def copyMiscFiles(self, destDir):
-        shutil.copy2(os.path.join(root,"license.txt"),destDir)
-
-    def compileIDL(self):
-	buildDir = os.path.join(self.bdist_base, "idl")
-	pattern = re.compile(r"(.*)\.idl$")
-	xpidl = os.path.join(IDL_TOOLS_PATH, "xpidl")
-	xpt_link = os.path.join(IDL_TOOLS_PATH, "xpt_link")
-
-	if not os.access(buildDir, os.F_OK):
-	    os.mkdir(buildDir)
-
-	idlDir = os.path.join(root, 'platform', platform, 'idl')
-	generatedFiles = []
-	if not os.access(idlDir, os.F_OK):
-	    self.typeLibrary = None
-	    return
-
-	for name in os.listdir(idlDir):
-	    m = pattern.match(name)
-	    if not m:
-		continue
-	    inPath = os.path.join(idlDir, name)
-	    outPath = os.path.join(buildDir, m.group(1) + ".xpt")
-
-	    result = os.spawnl(os.P_WAIT, xpidl, xpidl, "-m", "typelib",
-			       "-w", "-v", "-e", outPath,
-			       "-I", IDL_INCLUDE_PATH, inPath)
-	    if result != 0:
-		raise OSError, "Couldn't compile %s (error code %s)" % \
-		    (inPath, result)
-
-	    generatedFiles.append(outPath)
-
-	if len(generatedFiles) == 0:
-	    self.typeLibrary = None
-	    return
-
-	outXpt = os.path.join(self.bdist_base, 'xpcom_typelib.xpt')
-        log.info("linking IDL files to %s", outXpt)
-	result = os.spawnl(os.P_WAIT, xpt_link, xpt_link, outXpt,
-			   *generatedFiles)
-	if result != 0:
-	    raise OSError, "Couldn't link compiled IDL to %s (error code %s)" \
-		% (outXpt, )
-
-	self.typeLibrary = outXpt
-
-    # Fill the app.config.template, to generate the real app.config.
-    # NEEDS: Very sloppy. The new file is just dropped in the source tree
-    # next to the old one. This also initializes self.templateVars.
-    def makeAppConfig(self):
-        import util
-        revision = util.queryRevision(root)
-        if revision is None:
-            revision = "unknown"
-        else:
-            revision = "%s - %s" % revision
-
-        path = os.path.join(root, 'resources', 'app.config')
-        s = open("%s.template" % path, "rt").read()
-        s = string.Template(s).safe_substitute(APP_REVISION = revision, APP_PLATFORM = 'windows-xul')
-        f = open(path, "wt")
-        f.write(s)
-        f.close()
-
-        self.templateVars = util.readSimpleConfigFile(path)
-
-    def setTemplateVariable(self, key, value):
-        assert self.templateVars, \
-            "Must call makeAppConfig before setTemplateVariable"
-        self.templateVars[key] = value
-
-    def getTemplateVariable(self, key):
-        assert self.templateVars, \
-            "Must call makeAppConfig before getTemplateVariable"
-        return self.templateVars[key]
-
-    # Given a file <filename>.template, replace all applicable
-    # template variables and generate a file <filename> right next to
-    # it in the tree. 'Template variables' are anything in app.config,
-    # plus anything set with setTemplateVariable.
-    # NEEDS: same deal as makeAppConfig: sloppy; shouldn't drop files in
-    # the source tree.
-    def fillTemplate(self, filename):
-        assert self.templateVars, \
-            "Must call makeAppConfig before fillTemplate"
-        s = open("%s.template" % filename, "rt").read()
-        s = string.Template(s).safe_substitute(self.templateVars)
-        f = open(filename, "wt")
-        f.write(s)
-        f.close()
-
-    def fillTemplates(self):
-        xulBase = os.path.join(root, 'platform', platform, 'xul')
-
-        self.fillTemplate(os.path.join(xulBase, 'application.ini'))
-        self.fillTemplate(os.path.join(xulBase, 'defaults', 'preferences',
-                                       'prefs.js'))
-
-        # NEEDS: generalize to do the whole tree, so as to handle all
-        # locales
-        self.fillTemplate(os.path.join(xulBase, 'chrome', 'locale',
-                                       'en-US', 'main.dtd'))        
-        self.fillTemplate(os.path.join(xulBase, 'chrome', 'locale',
-                                       'en-US', 'about.dtd'))        
-        self.fillTemplate(os.path.join(xulBase, 'chrome', 'locale',
-                                       'en-US', 'bugreport.dtd'))        
-
-###############################################################################
-
-class runxul(Command, Common):
-    description = "test run of a Mozilla XUL-based application"
-
-    def __init__(self, *rest):
-        Command.__init__(self, *rest)
-        Common.__init__(self)
-
-    # List of option tuples: long name, short name (None if no short
-    # name), and help string.
-    user_options = [
-        ('bdist-base=', 'b',
-         'base directory for build library (default is build)'),
-        ('dist-dir=', 'd',
-         "directory to put final built distributions in (default is dist)"),
-	('bdist-dir=', 'd',
-	 "temporary directory for creating the distribution"),
-        ]
-
-    def initialize_options(self):
-        self.bdist_base = None
-	self.bdist_dir = None
-        self.dist_dir = 'dist'
-
-	self.childPythonPaths = []
-	self.childDLLPaths = []
-
-    def finalize_options(self):
-        if self.bdist_dir is None:
-            bdist_base = self.get_finalized_command('bdist').bdist_base
-            self.bdist_dir = os.path.join(bdist_base, 'xul')
-
-        self.set_undefined_options('bdist',
-                                   ('dist_dir', 'dist_dir'),
-                                   ('bdist_base', 'bdist_base'))
-
-	# Find our 'resources' tree (NEEDS)
-	self.appResources = os.path.join(root, 'resources')
-
-    def buildXulrunnerInstallation(self):
-	buildBase = os.path.join(self.bdist_base, "xulrunner")
-	self.xulrunnerDir = buildBase
-	markFile = os.path.join(buildBase, ".xulrunnerBuilt")
-	if os.access(markFile, os.F_OK):
-	    # Mark file exists. We take this to mean that there is
-	    # valid xulrunner tree already built in buildBase.
-	    return
-
-        log.info("assembling temporary xulrunner tree in %s" % buildBase)
-
-	# First, copy in the basic Xulrunner distribution.
-	copyTreeExceptSvn(XULRUNNER_DIR, buildBase)
-
-	# Then, copy the extra PyXPCOM files over it.
-	copyTreeExceptSvn(PYXPCOM_DIR, buildBase)
-
-        # Copy the license file over
-        # NEEDS: (huh? this doesn't belong here at all)
-        self.copyMiscFiles(self.bdist_base)
-
-        # Finally, drop in our plugins.
-        self.copyVLCPluginFiles(self.bdist_base, buildBase)
-
-	# Create the mark file to indicate that we now have a build.
-	open(markFile, 'w')
-
-    def run(self):
-        self.makeAppConfig()
-        self.setTemplateVariable("pyxpcomIsEmbedded", "false")
-        self.fillTemplates()                  
-
-        template_compiler.compileAllTemplates(root)
-
-        # Build extensions and add results to child search path
-        build = self.reinitialize_command('build')
-        build.build_base = self.bdist_base
-        build.run()
-        if build.build_platlib is not None:
-	    self.childPythonPaths.append(build.build_platlib)
-        if build.build_lib is not None:
-	    self.childPythonPaths.append(build.build_lib)
-
-	# Add application Python modules to child search path
-	self.childPythonPaths. \
-	    extend([
-		os.path.join(root, 'platform', platform),
-		os.path.join(root, 'platform'),
-		os.path.join(root, 'portable'),
-		])
-
-	# Put together an xulrunner installation that meets our standards.
-	self.buildXulrunnerInstallation()
-	xulrunnerBinary = os.path.join(self.xulrunnerDir, "xulrunner")
-
-	# Find application -- presently hardcoded to a tree in
-	# 'xul'.
-	self.applicationRoot = os.path.join(root, 'platform', platform,
-					    'xul')
-	applicationIni = os.path.join(self.applicationRoot, 'application.ini')
-
-	# Compile any IDL in the application
-        log.info("compiling type libraries")
-	self.compileIDL()
-
-	# Run the app. NEEDS: The main hack here is how we write the
-	# type library into the source tree, which is pretty odious.
-	if self.typeLibrary:
-	    # NEEDS: even if we did it this way, ensure that 'components'
-	    # exists
-	    typeLibraryPath = os.path.join(self.applicationRoot, 'components',
-					   'types.xpt')
-	    shutil.copy2(self.typeLibrary, typeLibraryPath)
-
-        self.buildDownloadDaemon(self.bdist_base)
-
-	newEnv = copy.deepcopy(os.environ)
-	oldPath = 'PATH' in newEnv and [newEnv['PATH']] or []
-	newEnv['PATH'] = \
-	    ';'.join(oldPath + self.childDLLPaths)
-	oldPyPath = 'PYTHONPATH' in newEnv and [newEnv['PYTHONPATH']] or []
-	newEnv['PYTHONPATH'] = \
-	    ';'.join(oldPyPath + self.childPythonPaths)
-	newEnv['RUNXUL_RESOURCES'] = self.appResources
-	print "Starting application"
-#	os.execle(xulrunnerBinary, xulrunnerBinary, applicationIni, newEnv)
-	os.execle(xulrunnerBinary, xulrunnerBinary, applicationIni, "-jsconsole", "-console", newEnv)
-
-###############################################################################
-
-class bdist_xul_dumb(Command, Common):
+class bdist_xul_dumb(Command):
     description = "build redistributable directory for Mozilla-based app"
 
     def __init__(self, *rest):
         Command.__init__(self, *rest)
-        Common.__init__(self)
+        self.templateVars = None
 
     # List of option tuples: long name, short name (None if no short
     # name), and help string.
@@ -477,18 +204,18 @@ class bdist_xul_dumb(Command, Common):
          'base directory for build library (default is build)'),
         ('dist-dir=', 'd',
          "directory to put final built distribution in (default is dist)"),
-	('bdist-dir=', 'd',
-	 "temporary directory for creating the distribution"),
+        ('bdist-dir=', 'd',
+         "temporary directory for creating the distribution"),
         ]
 
     def initialize_options(self):
         # NEEDS: allow 'includes' and 'excludes' options?
         self.bdist_base = None
-	self.bdist_dir = None
+        self.bdist_dir = None
         self.dist_dir = 'dist'
 
-	self.childPythonPaths = []
-	self.childDLLPaths = []
+        self.childPythonPaths = []
+        self.childDLLPaths = []
 
     def finalize_options(self):
         if self.bdist_dir is None:
@@ -499,14 +226,15 @@ class bdist_xul_dumb(Command, Common):
                                    ('dist_dir', 'dist_dir'),
                                    ('bdist_base', 'bdist_base'))
 
-	# Find our 'resources' tree (NEEDS)
-	self.appResources = os.path.join(root, 'resources')
+        # Find our 'resources' tree (NEEDS)
+        self.appResources = os.path.join(root, 'resources')
         # Find our various data bits (NEEDS)
-	self.xulTemplateDir = os.path.join(root, 'platform', platform,
+        self.xulTemplateDir = os.path.join(root, 'platform', platform,
                                            'xul')
 
     def run(self):
         self.makeAppConfig()
+        self.makeStubEXE()
         self.setTemplateVariable("pyxpcomIsEmbedded", "true")
         self.fillTemplates()                  
 
@@ -526,18 +254,18 @@ class bdist_xul_dumb(Command, Common):
         build.build_base = self.bdist_base
         build.run()
         if build.build_platlib is not None:
-	    packagePaths.append(build.build_platlib)
+            packagePaths.append(build.build_platlib)
         if build.build_lib is not None:
-	    packagePaths.append(build.build_lib)
+            packagePaths.append(build.build_lib)
 
-	# Add application Python modules to search path
+        # Add application Python modules to search path
         # NEEDS: should compile all Python files, and add the *build*
         # directories to the path.
-	packagePaths.extend([
+        packagePaths.extend([
                 os.path.join(root, 'platform', platform),
-		os.path.join(root, 'platform'),
-		os.path.join(root, 'portable'),
-		])
+                os.path.join(root, 'platform'),
+                os.path.join(root, 'portable'),
+                ])
 
         # Add PyXPCOM's runtime scripts directory to the search path,
         # and call out a list of top-level modules (those imported by
@@ -545,7 +273,7 @@ class bdist_xul_dumb(Command, Common):
         # cause the dependency scanner to conclude that all of the
         # PyXPCOM scripts are necessary.
         # NEEDS: again, should arrange to have these get built.
-	packagePaths.extend([
+        packagePaths.extend([
                 os.path.join(PYXPCOM_DIR, "python"),
                 ])
         moduleIncludes = [# Public to Python code
@@ -593,10 +321,10 @@ class bdist_xul_dumb(Command, Common):
         copyTreeExceptSvn(self.xulTemplateDir, self.dist_dir)
         log.info("assembling xulrunner")
         self.xulrunnerOut = os.path.join(self.dist_dir, 'xulrunner')
-	copyTreeExceptSvn(XULRUNNER_DIR, self.xulrunnerOut)
+        copyTreeExceptSvn(XULRUNNER_DIR, self.xulrunnerOut)
         # (Copy *only* the components part, not the Python part; that
         #  got sucked into the dependency scan above)
-	copyTreeExceptSvn(os.path.join(PYXPCOM_DIR, 'components'),
+        copyTreeExceptSvn(os.path.join(PYXPCOM_DIR, 'components'),
                           os.path.join(self.xulrunnerOut, 'components'))
 
         # Copy the license file over
@@ -608,7 +336,7 @@ class bdist_xul_dumb(Command, Common):
         # NEEDS: make IDL directory configurable
         log.info("compiling type libraries")
         self.compileIDL()
-	if self.typeLibrary:
+        if self.typeLibrary:
             componentDir = os.path.join(self.dist_dir, 'components')
             if not os.access(componentDir, os.F_OK):
                 os.makedirs(componentDir)
@@ -673,10 +401,20 @@ class bdist_xul_dumb(Command, Common):
         # Copy in our application's resources.
         log.info("copying application resources")
         copyTreeExceptSvn(self.appResources,
-                          os.path.join(self.dist_dir, 'resources'))
+                          os.path.join(self.dist_dir, 'resources'),
+                          filterOut=['unittest', 'testdata'])
         shutil.copy2("Democracy.nsi", self.dist_dir)
         shutil.copy2("Democracy.ico", self.dist_dir)
 
+
+        locale_dir = os.path.join (self.appResources, "locale")
+
+        for source in glob (os.path.join (locale_dir, "*.mo")):
+            lang = os.path.basename(source)[:-3]
+            dest = os.path.join (self.dist_dir, 'resources', 'locale', lang, "LC_MESSAGES", "democracyplayer.mo")
+            self.mkpath(os.path.dirname(dest))
+            self.copy_file(source, dest)
+
         # NEEDS: set permissions/attributes on everything uniformly?
 
         # Create the top-level executable, and rename
@@ -772,6 +510,164 @@ class bdist_xul_dumb(Command, Common):
 
         return manifest
 
+    def buildDownloadDaemon(self, baseDir):
+        print "building download daemon"
+        os.system("%s setup_daemon.py py2exe --dist-dir daemon --bundle-files 1" % PYTHON_BINARY)
+        shutil.copy2(os.path.join("daemon","Democracy_Downloader.exe"), baseDir)
+
+    # NEEDS: if you look at the usage of this function, we're dropping
+    # the plugin into the xulrunner plugin directory, rather than the
+    # app bundle plugin directory, which is the way you're "supposed"
+    # to do it so your app is cleanly separated from xulrunner.
+    def copyVLCPluginFiles(self, baseDir, xulrunnerDir):
+        destDir = os.path.join(xulrunnerDir, 'plugins')
+        if not os.access(destDir, os.F_OK):
+            os.mkdir(destDir)
+
+        pluginFiles = ['npvlc.dll', 'vlcintf.xpt']
+        for f in pluginFiles:
+            shutil.copy2(os.path.join(VLC_MOZ_PLUGIN_DIR, f), destDir)
+
+        vlcPluginDest = os.path.join(baseDir, "vlc-plugins")
+        if not os.access(vlcPluginDest, os.F_OK):
+            os.mkdir(vlcPluginDest)
+
+        vlcPlugins = os.listdir(VLC_PLUGINS_DIR)
+        for f in vlcPlugins:
+            if f[0] != '.' and f != '_svn':
+                shutil.copy2(os.path.join(VLC_PLUGINS_DIR, f), vlcPluginDest)
+
+    def copyMiscFiles(self, destDir):
+        shutil.copy2(os.path.join(root,"license.txt"),destDir)
+
+    def compileIDL(self):
+        buildDir = os.path.join(self.bdist_base, "idl")
+        pattern = re.compile(r"(.*)\.idl$")
+        xpidl = os.path.join(IDL_TOOLS_PATH, "xpidl")
+        xpt_link = os.path.join(IDL_TOOLS_PATH, "xpt_link")
+
+        if not os.access(buildDir, os.F_OK):
+            os.mkdir(buildDir)
+
+        idlDir = os.path.join(root, 'platform', platform, 'idl')
+        generatedFiles = []
+        if not os.access(idlDir, os.F_OK):
+            self.typeLibrary = None
+            return
+
+        for name in os.listdir(idlDir):
+            m = pattern.match(name)
+            if not m:
+                continue
+            inPath = os.path.join(idlDir, name)
+            outPath = os.path.join(buildDir, m.group(1) + ".xpt")
+
+            result = os.spawnl(os.P_WAIT, xpidl, xpidl, "-m", "typelib",
+                               "-w", "-v", "-e", outPath,
+                               "-I", IDL_INCLUDE_PATH, inPath)
+            if result != 0:
+                raise OSError, "Couldn't compile %s (error code %s)" % \
+                    (inPath, result)
+
+            generatedFiles.append(outPath)
+
+        if len(generatedFiles) == 0:
+            self.typeLibrary = None
+            return
+
+        outXpt = os.path.join(self.bdist_base, 'xpcom_typelib.xpt')
+        log.info("linking IDL files to %s", outXpt)
+        result = os.spawnl(os.P_WAIT, xpt_link, xpt_link, outXpt,
+                           *generatedFiles)
+        if result != 0:
+            raise OSError, "Couldn't link compiled IDL to %s (error code %s)" \
+                % (outXpt, )
+
+        self.typeLibrary = outXpt
+
+    # Fill the app.config.template, to generate the real app.config.
+    # NEEDS: Very sloppy. The new file is just dropped in the source tree
+    # next to the old one. This also initializes self.templateVars.
+    def makeAppConfig(self):
+        import util
+        revision = util.queryRevision(root)
+        if revision is None:
+            revision = "unknown"
+        else:
+            revision = "%s - %s" % revision
+
+        path = os.path.join(root, 'resources', 'app.config')
+        fillTemplate("%s.template" % path, path,
+            APP_REVISION = revision, APP_PLATFORM = 'windows-xul')
+        self.templateVars = util.readSimpleConfigFile(path)
+
+    def makeStubEXE(self):
+         version = self.getTemplateVariable('appVersion')
+         versionComponents = version.split('-')[0].split('.')
+         while len(versionComponents) > 4:
+             versionComponents.append('0')
+         versionWithCommas = ','.join(versionComponents)
+         stubDir = os.path.join(root, '..', 'dtv-binary-kit', 'stub')
+         template = os.path.join(stubDir, 'Democracy.rc.template')
+         outfile = os.path.join(stubDir, 'Democracy.rc')
+         fillTemplate(template, outfile,
+                 VERSION_WITH_COMMAS=versionWithCommas, VERSION=version)
+         olddir = os.getcwd()
+         os.chdir(stubDir)
+         rv = os.system("rc Democracy.rc")
+         if rv == 0:
+             rv = os.system("cl Democracy.cpp /link /subsystem:windows /machine:x86 Democracy.RES")
+         os.chdir(olddir)
+         if rv != 0:
+            raise OSError("Making stub exe failed")
+
+    def setTemplateVariable(self, key, value):
+        assert self.templateVars, \
+            "Must call makeAppConfig before setTemplateVariable"
+        self.templateVars[key] = value
+
+    def getTemplateVariable(self, key):
+        assert self.templateVars, \
+            "Must call makeAppConfig before getTemplateVariable"
+        return self.templateVars[key]
+
+    # Given a file <filename>.template, replace all applicable
+    # template variables and generate a file <filename> right next to
+    # it in the tree. 'Template variables' are anything in app.config,
+    # plus anything set with setTemplateVariable.
+    # NEEDS: same deal as makeAppConfig: sloppy; shouldn't drop files in
+    # the source tree.
+    def fillTemplate(self, filename):
+        assert self.templateVars, \
+            "Must call makeAppConfig before fillTemplate"
+        fillTemplate("%s.template" % filename, filename, **self.templateVars)
+
+    def fillTemplates(self):
+        xulBase = os.path.join(root, 'platform', platform, 'xul')
+
+        self.fillTemplate(os.path.join(xulBase, 'application.ini'))
+        self.fillTemplate(os.path.join(xulBase, 'defaults', 'preferences',
+                                       'prefs.js'))
+
+        for lang in glob (os.path.join (xulBase, 'chrome', 'locale', '*')):
+            if len(lang) >= 4 and lang[-4:] in ("_svn", ".svn"):
+                continue
+    	    # NEEDS: generalize to do the whole tree, so as to handle all
+    	    # templates
+    	    self.fillTemplate(os.path.join(lang, 'main.dtd'))        
+    	    self.fillTemplate(os.path.join(lang, 'about.dtd'))        
+    	    self.fillTemplate(os.path.join(lang, 'bugreport.dtd'))        
+
+class runxul(bdist_xul_dumb):
+    def run(self):
+        bdist_xul_dumb.run(self)
+        log.info("starting up democracy")
+        oldDir = os.getcwd()
+        os.chdir(self.dist_dir)
+        subprocess.call(["Democracy.exe", "application.ini", "-jsconsole",
+                "-console"])
+        os.chdir(oldDir)
+
 ###############################################################################
 
 ###############################################################################
@@ -792,15 +688,34 @@ class bdist_xul (bdist_xul_dumb):
             ]:
             nsisVars[nsisName] = self.getTemplateVariable(ourName)
 
+        # Hardcoded elsewhere in this file, so why not here too?
+        nsisVars['CONFIG_EXECUTABLE'] = "Democracy.exe"
+        nsisVars['CONFIG_ICON'] = "Democracy.ico"
+
+        # One stage installer
         outputFile = "%s-%s.exe" % \
             (self.getTemplateVariable('shortAppName'),
              self.getTemplateVariable('appVersion'),
              )
         nsisVars['CONFIG_OUTPUT_FILE'] = outputFile
+        nsisVars['CONFIG_TWOSTAGE'] = "No"
 
-        # Hardcoded elsewhere in this file, so why not here too?
-        nsisVars['CONFIG_EXECUTABLE'] = "Democracy.exe"
-        nsisVars['CONFIG_ICON'] = "Democracy.ico"
+        nsisArgs = ["/D%s=%s" % (k, v) for (k, v) in nsisVars.iteritems()]
+        nsisArgs.append(os.path.join(self.dist_dir, "Democracy.nsi"))
+
+        if os.access(outputFile, os.F_OK):
+            os.remove(outputFile)
+        if subprocess.call([NSIS_PATH] + nsisArgs) != 0:
+            print "ERROR creating the 1 stage installer, quitting"
+            return
+
+        # Two stage installer
+        outputFile = "%s-%s-twostage.exe" % \
+            (self.getTemplateVariable('shortAppName'),
+             self.getTemplateVariable('appVersion'),
+             )
+        nsisVars['CONFIG_OUTPUT_FILE'] = outputFile
+        nsisVars['CONFIG_TWOSTAGE'] = "Yes"
 
         nsisArgs = ["/D%s=%s" % (k, v) for (k, v) in nsisVars.iteritems()]
         nsisArgs.append(os.path.join(self.dist_dir, "Democracy.nsi"))
@@ -809,32 +724,74 @@ class bdist_xul (bdist_xul_dumb):
             os.remove(outputFile)
         subprocess.call([NSIS_PATH] + nsisArgs)
 
-def copyTreeExceptSvn(src, dest):
+        self.zipfile = zip.ZipFile(os.path.join (self.dist_dir, "Democracy-Contents-%s.zip" % (self.getTemplateVariable('appVersion'),)), 'w', zip.ZIP_DEFLATED)
+        self.addFile (nsisVars['CONFIG_EXECUTABLE'])
+	self.addFile (nsisVars['CONFIG_ICON'])
+	self.addFile ("Democracy_Downloader.exe")
+	self.addFile ("application.ini")
+	self.addFile ("msvcp71.dll")
+	self.addFile ("msvcr71.dll")
+	self.addFile ("python24.dll")
+	self.addFile ("boost_python-vc71-mt-1_33.dll")
+
+	self.addDirectory ("chrome")
+	self.addDirectory ("components")
+	self.addDirectory ("defaults")
+	self.addDirectory ("resources")
+	self.addDirectory ("vlc-plugins")
+	self.addDirectory ("xulrunner")
+
+	self.zipfile.close()
+
+    def addFile(self, filename):
+	length = len(self.dist_dir)
+	if filename[:length] == (self.dist_dir):
+	    filename = filename[length:]
+	    while len(filename) > 0 and (filename[0] == '/' or filename[0] == '\\'):
+	        filename = filename[1:]
+	print "Compressing %s" % (filename,)
+	self.zipfile.write (os.path.join (self.dist_dir, filename), filename)
+
+    def addDirectory (self, dirname):
+	for root, dirs, files in os.walk (os.path.join (self.dist_dir, dirname)):
+	    for name in files:
+		self.addFile (os.path.join (root, name))
+
+def copyTreeExceptSvn(src, dest, filterOut=None):
     """Copy the contents of the given source directory into the given
     destination directory, creating the destination directory first if
     necessary. Preserve all file attributes. If the source directory
     contains directories, recursively copy them as well. However,
-    under no circumstances copy a file or directory named '.svn'."""
+    under no circumstances copy a file or directory named '.svn'.  If
+    filterOut is given, it should be a list of strings, we won't copy any of
+    those either.
+    """
+
+    if filterOut is None:
+        filterOut = ['.svn', "_svn"]
+    else:
+        filterOut.extend(['.svn', "_svn"])
 
     names = os.listdir(src)
     if not os.access(dest, os.F_OK):
-	os.mkdir(dest)
+        os.mkdir(dest)
     for name in names:
-	if name == ".svn":
-	    continue
-	srcname = os.path.join(src, name)
-	destname = os.path.join(dest, name)
-	if os.path.isdir(srcname):
-	    copyTreeExceptSvn(srcname, destname)
-	else:
-	    shutil.copy2(srcname, destname)
+        if name in filterOut:
+            continue
+        srcname = os.path.join(src, name)
+        destname = os.path.join(dest, name)
+        if os.path.isdir(srcname):
+            copyTreeExceptSvn(srcname, destname, filterOut)
+        else:
+            shutil.copy2(srcname, destname)
 
-setup(
-    ext_modules = ext_modules,
-    cmdclass = {
-	'build_ext': build_ext,
-	'runxul': runxul,
-	'bdist_xul_dumb': bdist_xul_dumb,
-	'bdist_xul': bdist_xul,
-	}
-)
+if __name__ == "__main__":
+    setup(
+        ext_modules = ext_modules,
+        cmdclass = {
+            'build_ext': build_ext,
+            'runxul': runxul,
+            'bdist_xul_dumb': bdist_xul_dumb,
+            'bdist_xul': bdist_xul,
+            }
+    )
