@@ -10,6 +10,13 @@ function updateLabel() {
   elt.setAttribute("value", textIn);
 }
 
+function updateLabelStartupError() {
+  var py = Components.classes["@participatoryculture.org/dtv/pybridge;1"].
+        getService(Components.interfaces.pcfIDTVPyBridge);
+  var elt = document.getElementById("log");
+  elt.setAttribute("value", py.getStartupError());
+}
+
 function copyLogToClipboard() {
   var log = document.getElementById("log");
   var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
