@@ -13,54 +13,111 @@ import errno
 import re
 import socket
 import threading
+import traceback
 from urlparse import urlparse, urljoin
 from collections import deque
+from gtcache import gettext as _
 
-from BitTornado.clock import clock
+from clock import clock
 
 import httpauth
 import config
 import prefs
-import dialogs
-from download_utils import cleanFilename, parseURL
+from download_utils import cleanFilename, parseURL, defaultPort
+from xhtmltools import URLEncodeDict, multipartEncode
 import eventloop
 import util
 import sys
 import time
 
-class NotReadyToSendError(Exception):
-    pass
-class ConnectionError(Exception):
-    pass
-class HTTPError(Exception):
-    def __init__(self, description=None):
-        self.description = description
-    def __str__(self):
-        if self.description is not None:
-            return "%s: %s" % (self.__class__, self.description)
-        else:
-            return str(self.__class__)
-class BadStatusLine(HTTPError):
-    pass
-class BadHeaderLine(HTTPError):
-    pass
-class UnexpectedStatusCode(HTTPError):
-    pass
-class ServerClosedConnection(HTTPError):
-    pass
-class BadChunkSize(HTTPError):
-    pass
-class CRLFExpected(HTTPError):
-    pass
-class PipelinedRequestNeverStarted(HTTPError):
-    pass
-class BadRedirect(HTTPError):
-    pass
-class AuthorizationFailed(HTTPError):
-    pass
-class RequestCanceledError(HTTPError):
-    pass
+PIPELINING_ENABLED = True
+SOCKET_READ_TIMEOUT = 60
+SOCKET_INITIAL_READ_TIMEOUT = 30
+SOCKET_CONNECT_TIMEOUT = 15
 
+# This pattern matches all possible strings.  I promise.
+URIPattern = re.compile(r'^([^?]*/)?([^/?]*)/*(\?(.*))?$')
+
+class NetworkError(Exception):
+    """Base class for all errors that will be passed to errbacks from getURL
+    and friends.  NetworkErrors can be display in 2 ways:
+
+    getFriendlyDescription() -- short, newbie friendly description 
+    getLongDescription() -- detailed description
+    """
+
+    def __init__(self, shortDescription, longDescription=None):
+        if longDescription is None:
+            longDescription = shortDescription
+        self.friendlyDescription = _("Error: %s") % shortDescription
+        self.longDescription = longDescription
+
+    def getFriendlyDescription(self):
+        return self.friendlyDescription
+
+    def getLongDescription(self):
+        return self.longDescription
+
+    def __str__(self):
+        return "%s: %s -- %s" % (self.__class__,
+                self.getFriendlyDescription(), self.getLongDescription())
+
+class ConnectionError(NetworkError):
+    def __init__(self, errorMessage):
+        self.friendlyDescription = _("Can't connect")
+        self.longDescription = _("Connection Error: %s") % errorMessage
+
+class SSLConnectionError(ConnectionError):
+    def __init__(self):
+        self.friendlyDescription = _("Can't connect")
+        self.longDescription = _("SSL connection error")
+
+class HTTPError(NetworkError):
+    def __init__(self, longDescription):
+        NetworkError.__init__(self, _("HTTP error"), longDescription)
+class BadStatusLine(HTTPError):
+    def __init__(self, line):
+        HTTPError.__init__(self, _("Bad Status Line: %s" % line))
+class BadHeaderLine(HTTPError):
+    def __init__(self, line):
+        HTTPError.__init__(self, _("Bad Header Line: %s" % line))
+class BadChunkSize(HTTPError):
+    def __init__(self, line):
+        HTTPError.__init__(self, _("Bad Chunk size: %s" % line))
+class CRLFExpected(HTTPError):
+    def __init__(self, crlf):
+        HTTPError.__init__(self, _("Expected CRLF got: %r" % crlf))
+class ServerClosedConnection(HTTPError):
+    def __init__(self, host):
+        HTTPError.__init__(self, _('%s closed connection') % host)
+
+class UnexpectedStatusCode(HTTPError):
+    def __init__(self, code):
+        if code == 404:
+            self.friendlyDescription = _("File not found")
+            self.longDescription = _("Got 404 status code")
+        else:
+            HTTPError.__init__(self, _("Bad Status Code: %s" % code))
+
+class AuthorizationFailed(NetworkError):
+    def __init__(self):
+        NetworkError.__init__(self, _("Authorization failed"))
+
+class PipelinedRequestNeverStarted(NetworkError):
+    # User should never see this one
+    def __init__(self):
+        NetworkError.__init__(self, _("Internal Error"),
+                _("Pipeline request never started"))
+
+class ConnectionTimeout(NetworkError):
+    def __init__(self, host):
+        NetworkError.__init__(self, _('Timeout'),
+                _('Connection to %s timed out') % host)
+
+class MalformedURL(NetworkError):
+    def __init__(self, url):
+        NetworkError.__init__(self, _('Invalid URL'),
+                _('%s is not a valid URL') % url)
 
 def trapCall(object, function, *args, **kwargs):
     """Convenience function do a util.trapCall, where when = 'While talking to
@@ -151,8 +208,13 @@ class AsyncSocket(object):
         self.socket = None
         self.readCallback = None
         self.closeCallback = closeCallback
+        self.readTimeout = None
+        self.timedOut = False
         self.connectionErrback = None
+        self.disableReadTimeout = False
+        self.readSomeData = False
         self.name = ""
+        self.lastClock = None
 
     def __str__(self):
         if self.name:
@@ -160,6 +222,23 @@ class AsyncSocket(object):
         else:
             return "Unknown %s" % (type(self).__name__,)
 
+    # The complication in the timeout code is because creating and
+    # cancelling a timeout costs some memory (timeout is in memory
+    # until it goes off, even if cancelled.)
+    def startReadTimeout(self):
+        if self.disableReadTimeout:
+            return
+        self.lastClock = clock()
+        if self.readTimeout is not None:
+            return
+        self.readTimeout = eventloop.addTimeout(SOCKET_INITIAL_READ_TIMEOUT, self.onReadTimeout,
+                "AsyncSocket.onReadTimeout")
+
+    def stopReadTimeout(self):
+        if self.readTimeout is not None:
+            self.readTimeout.cancel()
+            self.readTimeout = None
+
     def openConnection(self, host, port, callback, errback):
         """Open a connection.  On success, callback will be called with this
         object.
@@ -170,27 +249,40 @@ class AsyncSocket(object):
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.socket.setblocking(0)
         self.connectionErrback = errback
+        def handleGetHostByNameException(e):
+            trapCall(self, errback, ConnectionError(e[1]))
         def onAddressLookup(address):
             if self.socket is None:
                 # the connection was closed while we were calling gethostbyname
                 return
             try:
-                self.socket.connect_ex((address, port))
-            except Exception, e:
-                trapCall(self, errback, e)
-            else:
+                rv = self.socket.connect_ex((address, port))
+            except socket.gaierror:
+                trapCall(self, errback, ConnectionError('gaierror'))
+                return
+            if rv in (0, errno.EINPROGRESS, errno.EWOULDBLOCK):
                 eventloop.addWriteCallback(self.socket, onWriteReady)
+                self.socketConnectTimeout =  eventloop.addTimeout(
+                        SOCKET_CONNECT_TIMEOUT, onWriteTimeout,
+                        "socket connect timeout")
+            else:
+                msg = errno.errorcode[rv]
+                trapCall(self, errback, ConnectionError(msg))
         def onWriteReady():
             eventloop.removeWriteCallback(self.socket)
+            self.socketConnectTimeout.cancel()
             rv = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
             if rv == 0:
                 trapCall(self, callback, self)
             else:
                 msg = errno.errorcode[rv]
-                trapCall(self, errback, ConnectionError((rv, msg)))
+                trapCall(self, errback, ConnectionError(msg))
             self.connectionErrback = None
-
-        eventloop.callInThread(onAddressLookup, errback,
+        def onWriteTimeout():
+            eventloop.removeWriteCallback(self.socket)
+            trapCall(self, errback, ConnectionTimeout(host))
+            self.connectionErrback = None
+        eventloop.callInThread(onAddressLookup, handleGetHostByNameException,
                 socket.gethostbyname, host)
 
     def acceptConnection(self, host, port, callback, errback):
@@ -211,10 +303,11 @@ class AsyncSocket(object):
     def closeConnection(self):
         if self.isOpen():
             eventloop.stopHandlingSocket(self.socket)
+            self.stopReadTimeout()
             self.socket.close()
             self.socket = None
             if self.connectionErrback is not None:
-                error = ConnectionError("Connection closed")
+                error = NetworkError(_("Connection closed"))
                 trapCall(self, self.connectionErrback, error)
                 self.connectionErrback = None
 
@@ -243,6 +336,7 @@ class AsyncSocket(object):
             raise ValueError("Socket not connected")
         self.readCallback = readCallback
         eventloop.addReadCallback(self.socket, self.onReadReady)
+        self.startReadTimeout()
 
     def stopReading(self):
         """Stop reading from the socket."""
@@ -251,6 +345,21 @@ class AsyncSocket(object):
             raise ValueError("Socket not connected")
         self.readCallback = None
         eventloop.removeReadCallback(self.socket)
+        self.stopReadTimeout()
+
+    def onReadTimeout(self):
+        if self.readSomeData:
+            timeout = SOCKET_READ_TIMEOUT
+        else:
+            timeout = SOCKET_INITIAL_READ_TIMEOUT
+
+        if clock() < self.lastClock + timeout:
+            self.readTimeout = eventloop.addTimeout(self.lastClock + timeout - clock(), self.onReadTimeout,
+                "AsyncSocket.onReadTimeout")
+        else:
+            self.readTimeout = None
+            self.timedOut = True
+            self.handleEarlyClose('read')
 
     def handleSocketError(self, code, msg, operation):
         if code in (errno.EWOULDBLOCK, errno.EINTR):
@@ -295,10 +404,12 @@ class AsyncSocket(object):
             self.handleReadData(data)
 
     def handleReadData(self, data):
+        self.startReadTimeout()
         if data == '':
             if self.closeCallback:
                 trapCall(self, self.closeCallback, self, socket.SHUT_RD)
         else:
+            self.readSomeData = True
             trapCall(self, self.readCallback, data)
 
     def handleEarlyClose(self, operation):
@@ -318,7 +429,7 @@ class AsyncSSLStream(AsyncSocket):
     def openConnection(self, host, port, callback, errback):
         def onSocketOpen(self):
             self.socket.setblocking(1)
-            eventloop.callInThread(onSSLOpen, errback, socket.ssl,
+            eventloop.callInThread(onSSLOpen, handleSSLError, socket.ssl,
                     self.socket)
         def onSSLOpen(ssl):
             if self.socket is None:
@@ -328,6 +439,8 @@ class AsyncSSLStream(AsyncSocket):
             self.ssl = ssl
             # finally we can call the actuall callback
             callback(self)
+        def handleSSLError(error):
+            errback(SSLConnectionError())
         super(AsyncSSLStream, self).openConnection(host, port, onSocketOpen,
                 errback)
 
@@ -491,9 +604,15 @@ class HTTPConnection(ConnectionHandler):
         self.pipelinedRequest = None
         self.closeCallback = closeCallback
         self.readyCallback = readyCallback
+        self.requestsFinished = 0
+        self.bytesRead = 0
         self.sentReadyCallback = False
         self.headerCallback = self.bodyDataCallback = None
 
+    def handleData(self, data):
+        self.bytesRead += len(data)
+        super(HTTPConnection, self).handleData(data)
+
     def closeConnection(self):
         super(HTTPConnection, self).closeConnection()
         if self.closeCallback is not None:
@@ -510,16 +629,16 @@ class HTTPConnection(ConnectionHandler):
         if self.pipelinedRequest is not None:
             errback = self.pipelinedRequest[1]
             trapCall(self, errback, PipelinedRequestNeverStarted())
+            self.pipelinedRequest = None
 
     def canSendRequest(self):
-        return self.state == 'ready'
         return (self.state == 'ready' or 
                 (self.state != 'closed' and self.pipelinedRequest is None and
-                    not self.willClose))
+                    not self.willClose and PIPELINING_ENABLED))
 
     def sendRequest(self, callback, errback, requestStartCallback=None,
             headerCallback=None, bodyDataCallback = None, method="GET",
-            path='/', headers=None):
+            path='/', headers=None, postVariables = None, postFiles = None):
         """Sending an HTTP Request.  callback will be called if the request
         completes normally, errback will be called if there is a network
         error.
@@ -543,19 +662,40 @@ class HTTPConnection(ConnectionHandler):
         If bodyDataCallback is given it will be called as we read in the data
         for the body.  Also, the connection won't store the body in memory,
         and the callback is called, it will be passed None for the body.
+
+        postVariables is a dictionary of variable names to values
+
+        postFiles is a dictionary of variable names to dictionaries
+        containing filename, mimetype, and handle attributes. Handle
+        should be an already open file handle.
         """
 
         if not self.canSendRequest():
-            raise NotReadyToSendError()
+            raise NetworkError(_("Unknown"), 
+                    _("Internal Error: Not ready to send"))
 
         if headers is None:
             headers = {}
         else:
             headers = headers.copy()
         headers['Host'] = self.host.encode('idna')
+        if self.port != defaultPort(self.scheme):
+            headers['Host'] += ':%d' % self.port
         headers['Accept-Encoding'] = 'identity'
 
-        self.sendRequestData(method, path, headers)
+        if (method == "POST" and postVariables is not None and
+                            len(postVariables) > 0 and postFiles is None):
+            postData = URLEncodeDict(postVariables)
+            headers['Content-Type'] = 'application/x-www-form-urlencoded'
+            headers['Content-Length'] = '%d' % len(postData)
+        elif method == "POST" and postFiles is not None:
+            (postData, boundary) = multipartEncode(postVariables, postFiles)
+            headers['Content-Type'] = 'multipart/form-data; boundary=%s' % boundary
+            headers['Content-Length'] = '%d' % len(postData)            
+        else:
+            postData = None
+
+        self.sendRequestData(method, path, headers, postData)
         args = (callback, errback, requestStartCallback, headerCallback,
                 bodyDataCallback, method, path, headers)
         if self.state == 'ready':
@@ -584,6 +724,7 @@ class HTTPConnection(ConnectionHandler):
         self.requestHeaders = headers
         self.headers = {}
         self.contentLength = self.version = self.status = self.reason = None
+        self.bytesRead = 0
         self.body = ''
         self.willClose = True 
         # Assume we will close, until we get the headers
@@ -593,12 +734,14 @@ class HTTPConnection(ConnectionHandler):
         self.sentReadyCallback = False
         self.changeState('response-status')
 
-    def sendRequestData(self, method, path, headers):
+    def sendRequestData(self, method, path, headers, data = None):
         sendOut = []
         sendOut.append('%s %s HTTP/1.1\r\n' % (method, path))
         for header, value in headers.items():
             sendOut.append('%s: %s\r\n' % (header, value))
         sendOut.append('\r\n')
+        if data is not None:
+            sendOut.append(data)
         self.sendData(''.join(sendOut))
 
     def onStatusData(self):
@@ -683,6 +826,7 @@ class HTTPConnection(ConnectionHandler):
         while line is not None:
             if line == '':
                 self.finishRequest()
+                break
             line = self.buffer.readline()
 
     def handleStatusLine(self, line):
@@ -727,7 +871,10 @@ class HTTPConnection(ConnectionHandler):
     def handleHeaderLine(self, line):
         if self.unparsedHeaderLine == '':
             if line == '':
-                self.startBody()
+                if self.status != 100:
+                    self.startBody()
+                else:
+                    self.changeState('response-status')
             elif ':' in line:
                 self.parseHeader(line)
             else:
@@ -750,7 +897,7 @@ class HTTPConnection(ConnectionHandler):
         value = value.strip()
         header = header.lstrip().lower()
         if value == '':
-            print "DTV: Warning: Bad Header from %s:%s%s (%s)" % (self.host, self.port, self.path, line)
+            print "DTV: Warning: Bad Header from %s://%s:%s%s (%s)" % (self.scheme, self.host, self.port, self.path, line)
         if header not in self.headers:
             self.headers[header] = value
         else:
@@ -781,14 +928,26 @@ class HTTPConnection(ConnectionHandler):
         self.chunked = (te.lower() == 'chunked')
 
     def findExpectedLength(self):
-        if self.headers.get('transfer-encoding', 'identity') == 'identity':
+        self.contentLength = None
+        if self.status == 416:
+            try:
+                contentRange = self.headers['content-range']
+            except KeyError:
+                pass
+            else:
+                m = re.search('bytes\s+\*/(\d+)', contentRange)
+                if m is not None:
+                    try:
+                        self.contentLength = int(m.group(1))
+                    except (ValueError, TypeError):
+                        pass
+        if (self.contentLength is None and 
+                self.headers.get('transfer-encoding') in ('identity', None)):
             try:
                 self.contentLength = int(self.headers['content-length'])
             except (ValueError, KeyError):
                 pass
-            if self.contentLength < 0:
-                self.contentLength = None
-        else:
+        if self.contentLength < 0:
             self.contentLength = None
 
     def decideWillClose(self):
@@ -806,8 +965,9 @@ class HTTPConnection(ConnectionHandler):
             self.willClose = False
 
     def finishRequest(self):
-        # figure out what the response was before we do things like start a
-        # pipelined response.
+        # calculate the response and and remember our callback.  They may
+        # change after we start a pielined response.
+        origCallback = self.callback 
         if self.bodyDataCallback:
             body = None
         elif self.chunked:
@@ -815,19 +975,19 @@ class HTTPConnection(ConnectionHandler):
         else:
             body = self.body
         response = self.makeResponse(body)
-
         if self.stream.isOpen():
             if self.willClose:
                 self.closeConnection()
                 self.changeState('closed')
             elif self.pipelinedRequest is not None:
-                self.startNewRequest(*self.pipelinedRequest)
+                req = self.pipelinedRequest
                 self.pipelinedRequest = None
+                self.startNewRequest(*req)
             else:
                 self.changeState('ready')
                 self.idleSince = clock()
-
-        trapCall(self, self.callback, response)
+        trapCall(self, origCallback, response)
+        self.requestsFinished += 1
         self.maybeSendReadyCallback()
 
     def makeResponse(self, body=None):
@@ -850,8 +1010,10 @@ class HTTPConnection(ConnectionHandler):
         if oldState == 'response-body' and self.contentLength is None:
             self.body = self.buffer.read()
             self.finishRequest()
+        elif self.stream.timedOut:
+            self.errback(ConnectionTimeout(self.host))
         else:
-            self.errback(ServerClosedConnection())
+            self.errback(ServerClosedConnection(self.host))
         self.checkPipelineNotStarted()
 
     def handleError(self, error):
@@ -932,7 +1094,8 @@ class HTTPConnectionPool(object):
         self.runPendingRequests()
 
     def addRequest(self, callback, errback, requestStartCallback,
-            headerCallback, bodyDataCallback, url, method, headers):
+            headerCallback, bodyDataCallback, url, method, headers,
+            postVariables = None, postFiles = None):
         """Add a request to be run.  The request will run immediately if we
         have a free connection, otherwise it will be queued.
 
@@ -941,7 +1104,7 @@ class HTTPConnectionPool(object):
 
         scheme, host, port, path = parseURL(url)
         if scheme not in ['http', 'https'] or host == '' or path == '':
-            errback (ValueError("Bad URL: %s" % (url,)))
+            errback (MalformedURL(url))
             return
         req = {
             'callback' : callback,
@@ -955,6 +1118,8 @@ class HTTPConnectionPool(object):
             'method': method,
             'path': path,
             'headers': headers,
+            'postVariables': postVariables,
+            'postFiles': postFiles,
         }
         self.pendingRequests.append(req)
         self.runPendingRequests()
@@ -976,7 +1141,7 @@ class HTTPConnectionPool(object):
                 conn.sendRequest(req['callback'], req['errback'],
                         req['requestStartCallback'], req['headerCallback'],
                         req['bodyDataCallback'], req['method'], req['path'],
-                        req['headers'])
+                        req['headers'], req['postVariables'], req['postFiles'])
             else:
                 conn = self._makeNewConnection(req)
             conns['active'].add(conn)
@@ -991,7 +1156,7 @@ class HTTPConnectionPool(object):
             conn.sendRequest(req['callback'], req['errback'],
                     req['requestStartCallback'], req['headerCallback'],
                     req['bodyDataCallback'], req['method'], req['path'],
-                    req['headers'])
+                    req['headers'], req['postVariables'], req['postFiles'])
         def openConnectionErrback(error):
             conns = self._getServerConnections(req['scheme'], req['host'], 
                     req['port'])
@@ -1054,7 +1219,7 @@ class HTTPClient(object):
 
     def __init__(self, url, callback, errback, headerCallback=None,
             bodyDataCallback=None, method="GET", start=0, etag=None,
-            modified=None, cookies={}):
+            modified=None, cookies={}, postVariables = None, postFiles = None):
         self.url = url
         self.callback = callback
         self.errback = errback
@@ -1072,6 +1237,8 @@ class HTTPClient(object):
                                # one or more of the following:
                                # 'Comment', 'CommentURL', 'origPath',
                                # 'origDomain', 'origPort'
+        self.postVariables = postVariables
+        self.postFiles = postFiles
         self.depth = 0
         self.authAttempts = 0
         self.updateURLOk = True
@@ -1160,12 +1327,11 @@ class HTTPClient(object):
         self.cancelled = False
         self.connection = None
         self.willHandleResponse = False
+        self.gotBadStatusCode = False
         if 'Authorization' not in self.headers:
             scheme, host, port, path = parseURL(self.redirectedURL)
             def callback(authHeader):
                 if self.cancelled:
-                    error = RequestCanceledError()
-                    self.errback(error)
                     return
                 if authHeader is not None:
                     self.headers["Authorization"] = authHeader
@@ -1182,9 +1348,22 @@ class HTTPClient(object):
         self.connectionPool.addRequest(self.callbackIntercept,
                 self.errbackIntercept, self.onRequestStart, self.onHeaders,
                 bodyDataCallback,
-                self.url, self.method, self.headers)
+                self.url, self.method, self.headers, self.postVariables,
+                self.postFiles)
+
+    def statusCodeExpected(self, status):
+        expectedStatusCodes = set([200])
+        if self.start != 0:
+            expectedStatusCodes.add(206)
+        if self.etag is not None or self.modified is not None:
+            expectedStatusCodes.add(304)
+        return status in expectedStatusCodes
 
     def callbackIntercept(self, response):
+        if self.cancelled:
+            print "WARNING: Callback on a cancelled request for %s" % self.url
+            traceback.print_stack()
+            return
         if self.shouldRedirect(response):
             self.handleRedirect(response)
         elif self.shouldAuthorize(response):
@@ -1195,11 +1374,7 @@ class HTTPClient(object):
         else:
             self.connection = None
             expectedStatusCodes = [200]
-            if self.start != 0:
-                expectedStatusCodes.append(206)
-            if self.etag is not None or self.modified is not None:
-                expectedStatusCodes.append(304)
-            if response['status'] in expectedStatusCodes:
+            if not self.gotBadStatusCode:
                 if self.callback:
                     response = self.prepareResponse(response)
                     trapCall(self, self.callback, response)
@@ -1208,14 +1383,23 @@ class HTTPClient(object):
                 self.errbackIntercept(error)
 
     def errbackIntercept(self, error):
-        self.connection = None
-        if isinstance(error, PipelinedRequestNeverStarted):
+        if self.cancelled:
+            return
+        elif isinstance(error, PipelinedRequestNeverStarted):
             # Connection closed before our pipelined request started.  RFC
             # 2616 says we should retry
             self.startRequest() 
             # this should give us a new connection, since our last one closed
-            return
-        trapCall(self, self.errback, error)
+        elif (isinstance(error, ServerClosedConnection) and
+                self.connection is not None and
+                self.connection.requestsFinished > 0 and 
+                self.connection.bytesRead == 0):
+            # Connection closed when trying to reuse an http connection.  We
+            # should retry with a fresh connection
+            self.startRequest()
+        else:
+            self.connection = None
+            trapCall(self, self.errback, error)
 
     def onRequestStart(self, connection):
         if self.cancelled:
@@ -1226,13 +1410,17 @@ class HTTPClient(object):
     def onHeaders(self, response):
         if self.shouldRedirect(response) or self.shouldAuthorize(response):
             self.willHandleResponse = True
-        elif self.headerCallback is not None:
-            response = self.prepareResponse(response)
-            if not trapCall(self, self.headerCallback, response):
-                self.cancel()
+        else:
+            if not self.statusCodeExpected(response['status']):
+                self.gotBadStatusCode = True
+            if self.headerCallback is not None:
+                response = self.prepareResponse(response)
+                if not trapCall(self, self.headerCallback, response):
+                    self.cancel()
 
     def onBodyData(self, data):
-        if not self.willHandleResponse and self.bodyDataCallback:
+        if (not self.willHandleResponse and not self.gotBadStatusCode and 
+                self.bodyDataCallback):
             if not trapCall(self, self.bodyDataCallback, data):
                 self.cancel()
 
@@ -1381,10 +1569,21 @@ class HTTPClient(object):
             filename = self.findValueFromHeader(disposition, 'filename')
             if filename is not None:
                 return cleanFilename(filename)
-        match = re.search("([^/]+)/?$", response['path'])
-        if match is not None:
-            return cleanFilename(match.group(1))
-        return 'unknown'
+        match = URIPattern.match(response['path'])
+        if match is None:
+            # This code path will never be executed.
+            return cleanFilename(response['path'])
+        filename = match.group(2)
+        query = match.group(4)
+        if not filename:
+            ret = query
+        elif not query:
+            ret = filename
+        else:
+            ret = "%s-%s" % (filename, query)
+        if ret is None:
+            ret = 'unknown'
+        return cleanFilename(ret)
 
     def getCharsetFromResponse(self, response):
         try:
@@ -1413,6 +1612,7 @@ class HTTPClient(object):
         if response['status'] == 303:
             # "See Other" we must do a get request for the result
             self.method = "GET"
+            self.postVariables = None
         if 'Authorization' in self.headers:
             del self.headers["Authorization"]
         self.startRequest()
@@ -1442,13 +1642,23 @@ class HTTPClient(object):
                 trapCall(self, self.errback, AuthorizationFailed())
         httpauth.askForHTTPAuth(callback, self.url, realm, authScheme)
 
+# Grabs a URL in the background using the eventloop
+# defaultMimeType is used for file:// URLs
 def grabURL(url, callback, errback, headerCallback=None,
         bodyDataCallback=None, method="GET", start=0, etag=None,
-        modified=None, cookies = {}):
-    client = HTTPClient(url, callback, errback, headerCallback,
-            bodyDataCallback, method, start, etag, modified, cookies)
-    client.startRequest()
-    return client
+        modified=None, cookies = {}, postVariables = None, postFiles = None,
+        defaultMimeType='application/octet-stream'):
+    if url.startswith("file://"):
+        callback({"body":file(url[7:]).read(),
+                      "updated-url":url,
+                      "redirected-url":url,
+                      "content-type": defaultMimeType,
+                   })
+    else:
+        client = HTTPClient(url, callback, errback, headerCallback,
+            bodyDataCallback, method, start, etag, modified, cookies, postVariables, postFiles)
+        client.startRequest()
+        return client
 
 class HTTPHeaderGrabber(HTTPClient):
     """Modified HTTPClient to get the headers for a URL.  It tries to do a
