@@ -26,19 +26,22 @@ process.
 
 import cPickle
 import os
-import shutil
 import traceback
+import shutil
 
 import config
 import database
-import databasesanity
 import databaseupgrade
-import olddatabaseupgrade
 import prefs
 import util
 import schema as schema_mod
 import eventloop
 import app
+import bsddb.db
+import dialogs
+from gtcache import gettext as _
+
+from clock import clock
 
 # FILEMAGIC should be the first portion of the database file.  After that the
 # file will contain pickle data
@@ -194,7 +197,10 @@ class ConverterBase(object):
         self.memory[id(object)] = convertedObject
 
         for name, schema in objectSchema.fields:
-            data = self.getSourceAttr(object, name)
+            try:
+                data = self.getSourceAttr(object, name)
+            except schema_mod.ValidationError, e:
+                self.handleValidationError(e, object, path, schema)
             try:
                 dataStr = str(data)
             except Exception, e:
@@ -359,6 +365,21 @@ def objectsToSavables(objects, objectSchemas=None):
     saver = SavableConverter(objectSchemas)
     return saver.convertObjectList(objects)
 
+oneSaver = SavableConverter()
+oneRestorer = SavableUnconverter()
+
+def objectToSavable(object):
+    """Transform a list of objects into something that we can save to disk.
+    This means converting any DDBObjects into SavebleObjects.
+    """
+
+    global oneSaver
+    if object.__class__ in oneSaver.classesToStrings:
+        oneSaver.memory = {}
+        return oneSaver.convertObject(object, schema_mod.SchemaObject(object.__class__), "")
+    else:
+        return None
+
 def savablesToObjects(savedObjects, objectSchemas=None):
     """Reverses the work of objectsToSavables"""
 
@@ -366,6 +387,18 @@ def savablesToObjects(savedObjects, objectSchemas=None):
     restorer.objectSchemas = objectSchemas
     return restorer.convertObjectList(savedObjects)
 
+def savableToObject(savedObject):
+    """Transform a list of objects into something that we can save to disk.
+    This means converting any DDBObjects into SavebleObjects.
+    """
+
+    global oneRestorer
+    oneRestorer.memory = {}
+    klass = oneRestorer.stringsToClasses[savedObject.classString]
+    object = oneRestorer.convertObject(savedObject, schema_mod.SchemaObject(klass), "")
+    oneRestorer.onPostConversion()
+    return object
+
 def saveObjectList(objects, pathname, objectSchemas=None, version=None):
     """Save a list of objects to disk."""
 
@@ -380,7 +413,7 @@ def saveObjectList(objects, pathname, objectSchemas=None, version=None):
     finally:
         f.close()
 
-def restoreObjectList(pathname, objectSchemas=None):
+def loadPickle(pathname, objectSchemas=None):
     """Restore a list of objects saved with saveObjectList."""
 
     f = open(pathname, 'rb')
@@ -399,72 +432,14 @@ def restoreObjectList(pathname, objectSchemas=None):
 
     return savablesToObjects(savedObjects, objectSchemas)
 
-backedUp = False
+def restoreObjectList(pathname, objectSchemas=None):
+    """Restore a list of objects saved with saveObjectList."""
 
-def saveDatabase(db=None, pathname=None):
-    """Save a database object."""
+    return loadPickle (pathname, objectSchemas)
 
-
-    if db is None:
-        db = database.defaultDatabase
-    if pathname is None:
-        pathname = config.get(prefs.DB_PATHNAME)
-
-    pathname = os.path.expanduser(pathname)
-
-    global backedUp
-    if (not backedUp):
-        try:
-            shutil.copyfile(pathname, pathname + '.bak')
-        except:
-            pass
-        else:
-            backedUp = True
-
-    tempPathname = pathname + '.temp'
-    try:
-        db.beginRead()
-        try:
-            objectsToSave = []
-            for o in db.objects:
-                objectsToSave.append(o[0])
-            databasesanity.checkSanity(objectsToSave)
-            saveObjectList(objectsToSave, tempPathname)
-        finally:
-            db.endRead()
-        try:
-            os.remove(pathname)
-        except:
-            pass
-        os.rename(tempPathname, pathname)
-    except IOError, err:
-        print "IOError Saving database: ", err
-        return err
-    except:
-        print "ERROR Saving database!"
-        traceback.print_exc()
-        return "Unknown"
-    return None
-
-def saveDatabaseIdle (db=None, pathname=None):
-    err = saveDatabase(db, pathname)
-    if err:
-        print "Save failed: %s" % (err,)
-        try:
-            err = err.strerror
-        except:
-            pass
-        app.delegate.saveFailed(err)
-    eventloop.addTimeout(300, saveDatabaseIdle, "Database Save", args=(db, pathname))
-
-def restoreDatabase(db=None, pathname=None, convertOnFail=True):
+def getObjects(pathname, convertOnFail):
     """Restore a database object."""
 
-    if db is None:
-        db = database.defaultDatabase
-    if pathname is None:
-        pathname = config.get(prefs.DB_PATHNAME)
-
     pathname = os.path.expanduser(pathname)
     if not os.path.exists(pathname):
         # maybe we crashed in saveDatabase() after deleting the real file, but
@@ -473,16 +448,23 @@ def restoreDatabase(db=None, pathname=None, convertOnFail=True):
         if os.path.exists(tempPathname):
             os.rename(tempPathname, pathname)
         else:
-            return # nope, there's no database to restore
+            return None # nope, there's no database to restore
 
+
+    global skipOnRestore
+    oldSkipOnRestore = skipOnRestore
+    skipOnRestore = True
     try:
         objects = restoreObjectList(pathname)
     except BadFileFormatError:
         if convertOnFail:
-            print "trying to convert database from old version"
+            if util.chatter:
+                print "trying to convert database from old version"
+            import olddatabaseupgrade
             olddatabaseupgrade.convertOldDatabase(pathname)
             objects = restoreObjectList(pathname)
-            print "*** Conversion Successfull ***"
+            if util.chatter:
+                print "*** Conversion Successfull ***"
         else:
             raise
     except ImportError, e:
@@ -500,10 +482,323 @@ def restoreDatabase(db=None, pathname=None, convertOnFail=True):
         else:
             raise
 
+    import databasesanity
     try:
-        databasesanity.checkSanity(objects)
+        databasesanity.checkSanity(objects, quiet=True, 
+                reallyQuiet=(not util.chatter))
     except databasesanity.DatabaseInsaneError, e:
         util.failedExn("When restoring database", e)
         # if the database fails the sanity check, try to restore it anyway.
         # It's better than notheing
-    db.restoreFromObjectList(objects)
+    skipOnRestore = oldSkipOnRestore
+    if not skipOnRestore:
+        for object in objects:
+            if hasattr(object, 'onRestore'):
+                object.onRestore()
+    return objects
+
+def restoreDatabase(db=None, pathname=None, convertOnFail=True):
+    if db is None:
+        db = database.defaultDatabase
+    if pathname is None:
+        pathname = config.get(prefs.DB_PATHNAME)
+
+    objects = getObjects (pathname, convertOnFail)
+    if objects:
+        db.restoreFromObjectList(objects)
+
+VERSION_KEY = "Democracy Version"
+
+class LiveStorage:
+    TRANSACTION_TIMEOUT = 10
+    TRANSACTION_NAME = "Save database"
+
+    def __init__(self, dbPath=None, restore=True):
+        try:
+            self.txn = None
+            self.dc = None
+            self.toUpdate = set()
+            self.toRemove = set()
+            self.errorState = False
+            if dbPath is not None:
+                self.dbPath = dbPath
+            else:
+                self.dbPath = config.get(prefs.BSDDB_PATHNAME)
+            start = clock()
+            self.openEmptyDB()
+            if restore:
+                try:
+                    try:
+                        self.db.open ("database")
+                        self.version = int(self.db[VERSION_KEY])
+                    except (bsddb.db.DBNoSuchFileError, KeyError):
+                        self.closeInvalidDB()
+                        try:
+                            restoreDatabase()
+                        except KeyboardInterrupt:
+                            raise
+                        except:
+                            print "WARNING: ERROR RESTORING OLD DATABASE"
+                            traceback.print_exc()
+                        self.saveDatabase()
+                    else:
+                        self.loadDatabase()
+                except KeyboardInterrupt:
+                    raise
+                except databaseupgrade.DatabaseTooNewError:
+                    raise
+                except:
+                    self.handleDatabaseLoadError()
+            else:
+                self.saveDatabase()
+            eventloop.addIdle(self.checkpoint, "Remove Unused Database Logs")
+            end = clock()
+            if end - start > 0.05 and util.chatter:
+                print "Database load slow: %.3f" % (end - start,)
+        except bsddb.db.DBNoSpaceError:
+            frontend.exit(28)
+
+    def handleDatabaseLoadError(self):
+        print "WARNING: exception while loading database"
+        traceback.print_exc()
+        self.closeInvalidDB()
+        self.dbenv.close()
+        self.saveInvalidDB()
+        self.openEmptyDB()
+        self.saveDatabase()
+
+    def saveInvalidDB(self):
+        dir = os.path.dirname(self.dbPath)
+        saveName = "corrupt_database"
+        i = 0
+        while os.path.exists(os.path.join(dir, saveName)):
+            i += 1
+            saveName = "corrupt_database.%d" % i
+
+        os.rename(self.dbPath, os.path.join(dir, saveName))
+
+    def openEmptyDB(self):
+        try:
+            os.makedirs(self.dbPath)
+        except KeyboardInterrupt:
+            raise
+        except:
+            pass
+        self.dbenv = bsddb.db.DBEnv()
+        self.dbenv.set_flags (bsddb.db.DB_AUTO_COMMIT | bsddb.db.DB_TXN_NOSYNC, True)
+        self.dbenv.set_lg_max (1024 * 1024)
+        self.dbenv.open (self.dbPath, bsddb.db.DB_INIT_LOG | bsddb.db.DB_INIT_MPOOL | bsddb.db.DB_INIT_TXN | bsddb.db.DB_RECOVER | bsddb.db.DB_CREATE)
+        self.db = bsddb.db.DB(self.dbenv)
+        self.closed = False
+
+    def closeInvalidDB(self):
+        try:
+            self.db.close()
+        except KeyboardInterrupt:
+            raise
+        except:
+            pass
+        self.db = None
+
+    def upgradeDatabase(self):
+        print "Upgrading database..."
+        savables = []
+        cursor = self.db.cursor()
+        while True:
+            next = cursor.next()
+            if next is None:
+                break
+            key, data = next
+            if key != VERSION_KEY:
+                try:
+                    savable = cPickle.loads(data)
+                    savables.append(savable)
+                except KeyboardInterrupt:
+                    raise
+                except:
+                    print data
+                    raise
+        cursor.close()
+        changed = databaseupgrade.upgrade(savables, self.version)
+        
+        txn = self.dbenv.txn_begin()
+        if changed is None:
+            self.rewriteDatabase(savables, txn)
+        else:
+            savables_set = set()
+            for o in savables:
+                savables_set.add(o)
+            for o in changed:
+                if o in savables_set:
+                    data = cPickle.dumps(o)
+                    self.db.put (str(o.savedData['id']), data, txn=txn)
+                else:
+                    try:
+                        self.db.delete (str(o.savedData['id']))
+                    except bsddb.db.DBNotFoundError:
+                        # If an object was created and removed during
+                        # upgrade, it won't be in the database to be
+                        # removed, so catch the exception
+                        pass
+        self.version = schema_mod.VERSION
+        self.db.put (VERSION_KEY, str(self.version), txn=txn)
+        txn.commit()
+        self.db.sync()
+
+        objects = savablesToObjects (savables)
+        db = database.defaultDatabase
+        db.restoreFromObjectList(objects)
+
+    def rewriteDatabase(self, savables, txn):
+        """Delete, then rewrite the entire database.  savables is a list of
+        SavableObjects that will be in the new database.  WARNING: This method
+        will probably take a long time.
+        """
+        print "Rewriting database"
+        cursor = self.db.cursor(txn=txn)
+        while True:
+            next = cursor.next()
+            if next is None:
+                break
+            cursor.delete()
+        cursor.close()
+        for o in savables:
+            data = cPickle.dumps(o)
+            self.db.put (str(o.savedData['id']), data, txn=txn)
+
+    def loadDatabase(self):
+        upgrade = (self.version != schema_mod.VERSION)
+        if upgrade:
+            return self.upgradeDatabase()
+        objects = []
+        cursor = self.db.cursor()
+        while True:
+            next = cursor.next()
+            if next is None:
+                break
+            key, data = next
+            if key != VERSION_KEY:
+                try:
+                    savable = cPickle.loads(data)
+                    object = savableToObject(savable)
+                    objects.append(object)
+                except KeyboardInterrupt:
+                    raise
+                except:
+                    print data
+                    raise
+        cursor.close()
+        db = database.defaultDatabase
+        db.restoreFromObjectList(objects)
+
+    def saveDatabase(self):
+        db = database.defaultDatabase
+        self.txn = self.dbenv.txn_begin()
+        self.db = bsddb.db.DB(self.dbenv)
+        self.db.open ("database", flags = bsddb.db.DB_CREATE, dbtype = bsddb.db.DB_HASH, txn=self.txn)
+        for o in db.objects:
+            self.update(o[0])
+        self.version = schema_mod.VERSION
+        self.db.put (VERSION_KEY, str(self.version), txn=self.txn)
+        self.txn.commit()
+        self.txn = None
+        self.db.sync()
+
+    def sync(self):
+        self.db.sync()
+
+    def close(self):
+        self.runUpdate()
+        self.closed = True
+        self.db.close()
+        self.dbenv.close()
+
+    def runUpdate(self):
+        try:
+            self.txn = self.dbenv.txn_begin()
+            for object in self.toRemove:
+                # If an object was created and removed between saves, it
+                # won't be in the database to be removed, so catch the
+                # exception
+                try:
+                    self.remove (object)
+                except bsddb.db.DBNotFoundError:
+                    pass
+            for object in self.toUpdate:
+                self.update (object)
+            self.txn.commit()
+            self.sync()
+            self.txn = None
+            self.dc = None
+            self.toUpdate = set()
+            self.toRemove = set()
+            if self.errorState:
+                title = _("%s database save succeeded") % (config.get(prefs.SHORT_APP_NAME), )
+                description = _("The database has been successfully saved. "
+                               "It is now safe to quit without losing any "
+                               "data.")
+                dialogs.MessageBoxDialog(title, description).run()
+                self.errorState = False
+        except bsddb.db.DBNoSpaceError, err:
+            if not self.errorState:
+                title = _("%s database save failed") % (config.get(prefs.SHORT_APP_NAME), )
+                description = _("%s was unable to save its database: Disk Full.\nWe suggest deleting files from the full disk or simply deleting some movies from your collection.\nRecent changes may be lost.") % (config.get(prefs.LONG_APP_NAME)) 
+                dialogs.MessageBoxDialog(title, description).run()
+                self.errorState = True
+            try:
+                self.txn.abort()
+            except KeyboardInterrupt:
+                raise
+            except:
+                # if we tried to do a commit and failed an abort doesn't work
+                pass
+            self.txn = None
+            self.dc = eventloop.addTimeout(self.TRANSACTION_TIMEOUT, self.runUpdate, self.TRANSACTION_NAME)
+            
+
+    def update (self, object):
+        if self.closed:
+            return
+        if self.txn is None:
+            self.toUpdate.add (object)
+            if self.dc is None:
+                self.dc = eventloop.addTimeout(self.TRANSACTION_TIMEOUT, self.runUpdate, self.TRANSACTION_NAME)
+        else:
+            savable = objectToSavable (object)
+            if savable:
+                key = str(object.id)
+                data = cPickle.dumps(savable)
+                self.db.put (key, data, txn=self.txn)
+
+    def remove (self, object):
+        if self.closed:
+            return
+        if self.txn is None:
+            self.toRemove.add (object)
+            try:
+                self.toUpdate.remove (object)
+            except KeyboardInterrupt:
+                raise
+            except:
+                pass
+            if self.dc is None:
+                self.dc = eventloop.addTimeout(self.TRANSACTION_TIMEOUT, self.runUpdate, self.TRANSACTION_NAME)
+        else:
+            self.db.delete (str(object.id), txn=self.txn)
+
+    def checkpoint (self):
+        try:
+            if self.closed:
+                return
+            self.dbenv.txn_checkpoint()
+            self.sync()
+            for logfile in self.dbenv.log_archive(bsddb.db.DB_ARCH_ABS):
+                try:
+                    os.remove(logfile)
+                except KeyboardInterrupt:
+                    raise
+                except:
+                    pass
+        except bsddb.db.DBNoSpaceError:
+            pass
+        eventloop.addTimeout(60, self.checkpoint, "Remove Unused Database Logs")
