@@ -13,7 +13,7 @@
 
 # Simply returns text
 def genRepeatText(varname, tid, prefix, text):
-    return '%s%s.write(%s)\n' % (prefix,varname, repr(text))
+    return '%s%s.write(%s)\n' % (prefix,varname, repr(toUTF8Bytes(text)))
 
 # Returns translated version of text
 def genRepeatTranslate(varname, tid, prefix, args):
@@ -23,34 +23,34 @@ def genRepeatTranslate(varname, tid, prefix, args):
     text = ' '.join(str(text).strip().split())
 
     if len(funcDict) == 0:
-        return '%s%s.write(_(%s))\n' % (prefix, varname, repr(text))
+        return '%s%s.write(toUTF8Bytes(_(%s)))\n' % (prefix, varname, repr(text))
     else:
         dictName = generateId()
         out = '%s%s = {}\n' % (prefix, dictName)
         for name in funcDict:
             temp = generateId()
-            out = '%s%s%s = IOBuffer()\n' % (out, prefix, temp)
+            out = '%s%s%s = StringIO()\n' % (out, prefix, temp)
             for (func, fargs) in funcDict[name]:
                 out = '%s%s' % (out, func(temp, tid, prefix, fargs))
-            out = '%s%s%s.close()\n' % (out, prefix, temp)
+            out = '%s%s%s.seek(0)\n' % (out, prefix, temp)
             out = '%s%s%s[%s] = %s.read()\n' % (out, prefix, dictName, repr(str(name)), temp)
 
-        out = '%s%s%s.write(Template(_(%s)).substitute(%s))\n' % (
+        out = '%s%s%s.write(Template(toUTF8Bytes(_(%s))).substitute(%s))\n' % (
             out, prefix, varname, repr(text), dictName)
         return out
 # Returns text if function does not evaluate to true
 def genRepeatTextHide(varname, tid, prefix, args):
     (ifValue, text) = args
     out = '%sif not (%s):\n'%(prefix, ifValue)
-    out = '%s%s    %s.write(%s)\n' % (out, prefix, varname, repr(text))
+    out = '%s%s    %s.write(%s)\n' % (out, prefix, varname, repr(toUTF8Bytes(text)))
     return out
         
 def genQuoteAttr(varname, tid, prefix, value):
-    return '%s%s.write(quoteattr(urlencode(toUni(%s))))\n'%(
+    return '%s%s.write(quoteattr(urlencode(toUTF8Bytes(%s))))\n'%(
         prefix, varname, value)
 
 def genRawAttr(varname, tid, prefix, value):
-    return '%s%s.write(quoteattr(toUni(%s)))\n'%(prefix, varname, value)
+    return '%s%s.write(quoteattr(toUTF8Bytes(%s)))\n'%(prefix, varname, value)
 
 # Adds a tid attribute to a tag and closes it
 def genRepeatTID(varname, tid, prefix, args):
@@ -58,11 +58,11 @@ def genRepeatTID(varname, tid, prefix, args):
 
 # Evaluates key with data
 def genRepeatEvalEscape(varname, tid, prefix, replace):
-    return '%s%s.write(escape(toUni(%s)))\n' % (prefix, varname, replace)
+    return '%s%s.write(escape(toUTF8Bytes(%s)))\n' % (prefix, varname, replace)
 
 # Evaluates key with data
 def genRepeatEval(varname, tid, prefix, replace):
-    return '%s%s.write(toUni(%s))\n' % (prefix, varname, replace)
+    return '%s%s.write(%s)\n' % (prefix, varname, replace)
 
 # Returns include iff function does not evaluate to true
 def genRepeatIncludeHide(varname, tid, prefix, args):
@@ -71,7 +71,7 @@ def genRepeatIncludeHide(varname, tid, prefix, args):
     text = f.read()
     f.close()
     out = '%sif not (%s):\n'%(prefix, ifValue)
-    out = '%s%s    %s.write(%s)\n' % (out, prefix, varname, repr(text))
+    out = '%s%s    %s.write(%s)\n' % (out, prefix, varname, repr(toUTF8Bytes(text)))
     return out
 
 def genHideSection(varname, tid, prefix, args):
@@ -99,9 +99,12 @@ def genUpdateHideOnView(varname, tid, prefix, args):
         out, prefix, repr(nodeId), viewName)
     return out
 
+def genInsertBodyTagExtra(varname, tid, prefix, args):
+    return '%s%s.write(" " + bodyTagExtra)\n' % (prefix, varname)
+
 from xml import sax
 from xhtmltools import toUTF8Bytes
-from StringIO import StringIO
+from cStringIO import StringIO
 from templatehelper import quoteattr, escape, HTMLPattern, attrPattern, resourcePattern, rawAttrPattern, generateId
 import re
 import os
@@ -162,6 +165,7 @@ def findTemplates(tpath):
     folders = []
     for template in os.listdir(tpath):
         if (template.find('.svn') == -1 and
+            template.find('_svn') == -1 and
             not (template.startswith('.') or template.startswith('#') or 
                  template.endswith('~') or template.endswith('.js') or 
                  template.endswith('.html'))):
@@ -177,7 +181,7 @@ def modifiedTime(dir):
     try:
         for (dirpath, dirnames, filenames) in os.walk(dir):
             for f in filenames:
-                if -1 == dirpath.find('.svn'):
+                if -1 == dirpath.find('.svn') and -1 == dirpath.find('_svn'):
                     t = os.stat(os.path.join(dirpath, f)).st_mtime
                     if t > maxTime:
                         maxTime = t
@@ -259,9 +263,9 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
     def render(self, fileobj):
         fileobj.write('# This is a generated file. Do not edit.\n')
         fileobj.write('from template import Handle, fillAttr, quoteAndFillAttr\n')
-        fileobj.write('from IOBuffer import IOBuffer\n')
-        fileobj.write('from xhtmltools import urlencode\n')
-        fileobj.write('from templatehelper import quoteattr, escape, toUni\n')
+        fileobj.write('from cStringIO import StringIO\n')
+        fileobj.write('from xhtmltools import urlencode, toUTF8Bytes\n')
+        fileobj.write('from templatehelper import quoteattr, escape\n')
         fileobj.write('from string import Template\n')
         fileobj.write('import app\n')
         fileobj.write('import views\n')
@@ -269,20 +273,20 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
         fileobj.write('import indexes\n')
         fileobj.write('import filters\n')
         fileobj.write('import resource\n')
-        fileobj.write('import gettext\n')
-        fileobj.write('_ = gettext.gettext\n')
-        fileobj.write('def fillTemplate(domHandler, dtvPlatform, eventCookie):\n')
+        fileobj.write('import gtcache\n')
+        fileobj.write('_ = gtcache.gettext\n')
+        fileobj.write('def fillTemplate(domHandler, dtvPlatform, eventCookie, bodyTagExtra, *args, **kargs):\n')
         self.handle.render(fileobj)
-        fileobj.write('\n\n    out = IOBuffer()\n')
+        fileobj.write('\n\n    out = StringIO()\n')
         
         if not self.onlyBody:
-            fileobj.write('    out.write("<?xml version=\\\"1.0\\\" encoding=\\\"UTF-8\\\"?>\\n<!DOCTYPE html PUBLIC \\\"-//W3C//DTD XHTML 1.0 Strict//EN\\\" \\\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\\\">\\n")\n')
+            fileobj.write('    out.write("<?xml version=\\\"1.0\\\" encoding=\\\"utf-8\\\"?>\\n<!DOCTYPE html PUBLIC \\\"-//W3C//DTD XHTML 1.0 Strict//EN\\\" \\\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\\\">\\n")\n')
             
         for count in range(len(self.outputLists[0])):
             (func, args) = self.outputLists[0][count]
             fileobj.write(func('out','','    ',args))
 
-        fileobj.write('    out.close()\n')        
+        fileobj.write('    out.seek(0)\n')        
         fileobj.write('\n\n    return (out, handle)\n')
 
     def returnIf(self,bool,value):
@@ -297,6 +301,7 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
         self.inInclude = False
         self.inRepeatView = False
         self.inUpdateView = False
+        self.inConfigUpdate = False
         self.inReplace = False
         self.inStaticReplace = False
         self.inExecOnUnload = False
@@ -350,18 +355,13 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
             pass
         elif 't:repeatForView' in attrs.keys():
             self.startRepeat(attrs['t:repeatForView'])
-            self.addText('<%s'%name)
-            for key in attrs.keys():
-                if key != 't:repeatForView':
-                    self.addAttr(key,attrs[key])
-            self.addIdAndClose()
+            self.addElementStart(name, attrs, addId=True)
         elif 't:updateForView' in attrs.keys():
             self.startUpdate(attrs['t:updateForView'])
-            self.addText('<%s'%name)
-            for key in attrs.keys():
-                if key != 't:updateForView':
-                    self.addAttr(key,attrs[key])
-            self.addIdAndClose()
+            self.addElementStart(name, attrs, addId=True)
+        elif 't:updateForConfigChange' in attrs.keys():
+            self.startConfigUpdate()
+            self.addElementStart(name, attrs, addId=True)
         elif 't:hideIf' in attrs.keys():
             ifValue = attrs['t:hideIf']
             if attrs.has_key('t:updateHideOnView'):
@@ -370,20 +370,20 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
                 self.addUpdateHideOnView(attrs['t:updateHideOnView'],name, ifValue, attrs)
             else:
                 self.startHiding(ifValue)
-
-                self.addText('<%s'%name)
-
-                for key in attrs.keys():
-                    if (key not in ['t:hideIf']):
-                        self.addAttr(key,attrs[key])
-                self.addText('>')
+                self.addElementStart(name, attrs)
                 #FIXME: support i18n tags within a t:hideIf
+        elif 't:showIf' in attrs.keys():
+            ifValue = "not (%s)" % attrs['t:showIf']
+            if attrs.has_key('t:updateHideOnView'):
+                if self.inRepeatView or self.inUpdateView:
+                    print "Warning: t:updateHideOnView is unsupported inside a repeat view"
+                self.addUpdateHideOnView(attrs['t:updateHideOnView'],name, ifValue, attrs)
+            else:
+                self.startHiding(ifValue)
+                self.addElementStart(name, attrs)
+                #FIXME: support i18n tags within a t:showIf
         elif 'i18n:translate' in attrs.keys():
-            self.addText('<%s'%name)
-            for key in attrs.keys():
-                if (not key.startswith('i18n:')):
-                    self.addAttr(key,attrs[key])
-            self.addText('>')
+            self.addElementStart(name, attrs)
             self.translateDepth.append(self.depth)
             self.translateText.append('')
             self.translateDict.append({})
@@ -395,21 +395,13 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
             self.addInclude(attrs['filename'])
 
         elif 't:replace' in attrs.keys():
-                self.addText('<%s'%name)
-                for key in attrs.keys():
-                    if key != 't:replace':
-                        self.addAttr(key,attrs[key])
-                self.addText('>')
+                self.addElementStart(name, attrs)
                 replace = attrs['t:replace']
                 self.addEvalEscape(replace)
                 self.inReplace = True
                 self.replaceDepth = self.depth      
         elif 't:replaceMarkup' in attrs.keys():
-                self.addText('<%s'%name)
-                for key in attrs.keys():
-                    if key != 't:replaceMarkup':
-                        self.addAttr(key,attrs[key])
-                self.addText('>')
+                self.addElementStart(name, attrs)
                 replace = attrs['t:replaceMarkup']
                 self.addEval(replace)
                 self.inReplace = True
@@ -440,12 +432,7 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
         elif name == 't:triggerActionOnUnload':
             self.handle.addTriggerActionURLOnUnload(attrs['url'])
         else:
-            self.addText('<%s'%name)
-            for key in attrs.keys():
-                if (not (key.startswith('t:') or key.startswith('i18n:')) or
-                        key == 't:contextMenu'):
-                    self.addAttr(key,attrs[key])
-            self.addText('>')
+            self.addElementStart(name, attrs)
 
     def endElement(self,name):
         if not self.started:
@@ -490,6 +477,13 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
             repeatId = generateId()
             self.addText('<span id="%s"/>'%quoteattr(repeatId))
             self.handle.addUpdate(repeatId, 'nextSibling', repeatList, self.repeatName)
+        elif self.inConfigUpdate and self.depth == self.repeatDepth:
+            self.addText('</%s>'%name)
+            self.endText()
+            repeatList = self.endConfigUpdate()
+            repeatId = generateId()
+            self.addText('<span id="%s"/>'%quoteattr(repeatId))
+            self.handle.addConfigUpdate(repeatId, 'nextSibling', repeatList)
         elif (len(self.translateDepth) > 0 and
                                      self.depth == self.translateDepth[-1]):
             self.addTranslation()
@@ -565,6 +559,12 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
         self.repeatName = name
         self.outputLists.append([])
 
+    def startConfigUpdate(self):
+        self.endText()
+        self.inConfigUpdate = True
+        self.repeatDepth = self.depth
+        self.outputLists.append([])
+
     def endRepeat(self):
         self.inRepeatView = False
         return self.outputLists.pop()
@@ -573,6 +573,10 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
         self.inUpdateView = False
         return self.outputLists.pop()
 
+    def endConfigUpdate(self):
+        self.inConfigUpdate = False
+        return self.outputLists.pop()
+
     def startHiding(self,ifValue):
         self.endText()
         self.hidingParams.append(ifValue)
@@ -591,8 +595,18 @@ class TemplateContentCompiler(sax.handler.ContentHandler):
     def addText(self, text):
         self.outputText.append( text)
 
-    def addTextHide(self,ifValue,text):
-        self.addInstruction(genRepeatTextHide,(ifValue,text))
+    def addElementStart(self, name, attrs, addId=False):
+        self.addText('<%s'%name)
+        for key in attrs.keys():
+            if (not (key.startswith('t:') or key.startswith('i18n:')) or
+                    key == 't:contextMenu'):
+                self.addAttr(key,attrs[key])
+        if name.lower() == 'body':
+	    self.addInstruction(genInsertBodyTagExtra, None)
+        if addId:
+            self.addIdAndClose()
+        else:
+            self.addText('>')
 
     def addTextEscape(self, text):
         self.outputText.append( escape(text))
@@ -669,6 +683,7 @@ class MetaHandle:
     def __init__(self):
         self.trackedViews = []
         self.updateRegions = []
+        self.configUpdateRegions = []
         self.subHandles = []
         self.triggerActionURLsOnLoad = []
         self.triggerActionURLsOnUnload = []
@@ -712,6 +727,10 @@ class MetaHandle:
     def addUpdate(self, anchorId, anchorType, templateFuncs, name):
         ur = (anchorId, anchorType, templateFuncs, name)
         self.updateRegions.append(ur)
+
+    def addConfigUpdate(self, anchorId, anchorType, templateFuncs):
+        ur = (anchorId, anchorType, templateFuncs)
+        self.configUpdateRegions.append(ur)
     
     def addSubHandle(self, handle):
         self.subHandles.append(handle)
@@ -736,34 +755,49 @@ class MetaHandle:
             fileobj.write('%s%s = Handle(domHandler, locals(), onUnlink = _execOnUnload)%s%s' % (prefix, varname, ending, ending))
         else:
             fileobj.write('%s%s = Handle(domHandler, locals(), onUnlink = lambda:None)%s%s' % (prefix, varname, ending, ending))
+
         count = 0
-        for tv in self.trackedViews:
-            (anchorId, anchorType, templateFuncs, name) = tv
-            repFunc = "rep_%s_%s" % (count, varname)
-            fileobj.write('%sdef %s(this, viewName, tid):%s' % (prefix, repFunc,ending))
-            fileobj.write('%s    out = IOBuffer()%s' % (prefix, ending))
-            for count2 in range(len(templateFuncs)):
-                (func, args) = templateFuncs[count2]
-                fileobj.write(func('out','',prefix+'    ',args))
-            fileobj.write('%s    out.close()%s' % (prefix, ending))
-            fileobj.write('%s    return out%s' % (prefix, ending))
-
-            fileobj.write('%s%s.addView(%s,%s,%s,%s, %s)%s' % (prefix, varname, repr(anchorId),repr(anchorType),name,repFunc,repr(name),ending))
-            count += 1
-
         for ur in self.updateRegions:
             (anchorId, anchorType, templateFuncs, name) = ur
             upFunc = "up_%s_%s" % (count, varname)
-            fileobj.write('%sdef %s(viewName, tid):%s' % (prefix, upFunc,ending))
-            fileobj.write('%s    out = IOBuffer()%s' % (prefix, ending))
+            fileobj.write('%sdef %s(viewName, view, tid):%s' % (prefix, upFunc,ending))
+            fileobj.write('%s    out = StringIO()%s' % (prefix, ending))
             for count2 in range(len(templateFuncs)):
                 (func, args) = templateFuncs[count2]
                 fileobj.write(func('out','',prefix+'    ',args))
-            fileobj.write('%s    out.close()%s' % (prefix, ending))
+            fileobj.write('%s    out.seek(0)%s' % (prefix, ending))
             fileobj.write('%s    return out%s' % (prefix, ending))
 
             fileobj.write('%s%s.addUpdate(%s,%s,%s,%s, %s)%s' % (prefix, varname, repr(anchorId),repr(anchorType),name,upFunc,repr(name),ending))
             count += 1
+
+        for ur in self.configUpdateRegions:
+            (anchorId, anchorType, templateFuncs) = ur
+            upFunc = "config_up_%s_%s" % (count, varname)
+            fileobj.write('%sdef %s(tid):%s' % (prefix, upFunc, ending))
+            fileobj.write('%s    out = StringIO()%s' % (prefix, ending))
+            for count2 in range(len(templateFuncs)):
+                (func, args) = templateFuncs[count2]
+                fileobj.write(func('out','',prefix+'    ',args))
+            fileobj.write('%s    out.seek(0)%s' % (prefix, ending))
+            fileobj.write('%s    return out%s' % (prefix, ending))
+
+            fileobj.write('%s%s.addConfigUpdate(%s,%s,%s)%s' % (prefix, varname, repr(anchorId),repr(anchorType),upFunc,ending))
+            count += 1
+
+        for tv in self.trackedViews:
+            (anchorId, anchorType, templateFuncs, name) = tv
+            repFunc = "rep_%s_%s" % (count, varname)
+            fileobj.write('%sdef %s(this, viewName, view, tid):%s' % (prefix, repFunc,ending))
+            fileobj.write('%s    out = StringIO()%s' % (prefix, ending))
+            for count2 in range(len(templateFuncs)):
+                (func, args) = templateFuncs[count2]
+                fileobj.write(func('out','',prefix+'    ',args))
+            fileobj.write('%s    out.seek(0)%s' % (prefix, ending))
+            fileobj.write('%s    return out%s' % (prefix, ending))
+
+            fileobj.write('%s%s.addView(%s,%s,%s,%s, %s)%s' % (prefix, varname, repr(anchorId),repr(anchorType),name,repFunc,repr(name),ending))
+            count += 1
             
         for action in self.triggerActionURLsOnLoad:
             fileobj.write('%s%s.addTriggerActionURLOnLoad(fillAttr(%s,locals()))%s' %
