@@ -5,34 +5,50 @@
 # by the entension module. This place seems as good as any.
 import cPickle #for database.pyx
 
-import gettext
 import inspect
 import shutil
 import re
-import types
 import traceback
 import random
+import types
 from xhtmltools import toUTF8Bytes, urlencode
 
 HTMLPattern = re.compile("^.*<body.*?>(.*)</body\s*>", re.S)
 attrPattern = re.compile("^(.*?)@@@(.*?)@@@(.*)$")
 resourcePattern = re.compile("^resource:(.*)$")
 rawAttrPattern = re.compile("^(.*)\*\*\*(.*?)\*\*\*(.*)$")
-evalCache = {}
+
+_unicache = {}
+_escapecache = {}
 
 def quoteattr(orig):
     return orig.replace('"','&quot;')
 
 def escape(orig):
-    return unicode(orig).replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
+    global _escapecache
+    try:
+        return _escapecache[orig]
+    except:
+        _escapecache[orig] = orig.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
+        return _escapecache[orig]
 
-def toUni(orig):
-    if type(orig) == types.IntType:
-        return "%d" % orig
-    else:
-        orig = toUTF8Bytes(orig)
-        return unicode(orig,'utf-8')
 
+def toUni(orig, encoding = None):
+    global _unicache
+    try:
+        return _unicache[orig]
+    except:
+        if isinstance(orig, types.UnicodeType):
+            # Let's not bother putting this in the cache.  Calculating
+            # it is very fast, and since this is a very common case,
+            # not caching here should help with memory usage.
+            return orig
+        elif not isinstance(orig, types.StringType):
+            _unicache[orig] = unicode(orig)
+        else:
+            orig = toUTF8Bytes(orig, encoding)
+            _unicache[orig] = unicode(orig,'utf-8')
+        return _unicache[orig]
 
 # Generate an arbitrary string to use as an ID attribute.
 def generateId():
