@@ -17,4 +17,9 @@ from test.schedulertest import *
 from test.httpclienttest import *
 from test.httpdownloadertest import *
 from test.feedparsertest import *
+from test.parseurltest import *
+from test.utiltest import *
+from test.playlisttest import *
+from test.unicodetest import *
+from test.databaseupgradetest import *
 import test.bmachinetest
