@@ -5,12 +5,22 @@ from os.path import expanduser
 import random
 import config
 import prefs
+import os
+import random
+import shutil
 import time
+import tempfile
 import storedatabase
+import fasttypes
 from threading import Thread
 
 from test.framework import DemocracyTestCase
 
+class SortableObject(database.DDBObject):
+    def __init__(self, value):
+        self.value = value
+        database.DDBObject.__init__(self)
+
 class EmptyViewTestCase(DemocracyTestCase):
     def setUp(self):
         DemocracyTestCase.setUp(self)
@@ -263,6 +273,21 @@ class SortTestCase(DemocracyTestCase):
         self.assertEqual(b,self.y)
         self.assertNotEqual(a,b)
         self.assertEqual(c,None)
+    def testSignalChange(self):
+        self.higher = self.y
+        self.assertEqual(self.sorted[0],self.y)
+        self.assertEqual(self.sorted[1],self.x)
+        self.x.signalChange()
+        self.assertEqual(self.sorted[0],self.y)
+        self.assertEqual(self.sorted[1],self.x)
+    def testResort(self):
+        self.sorted = self.everything.sort(self.sortFunc, resort=True)
+        self.assertEqual(self.sorted[0],self.y)
+        self.assertEqual(self.sorted[1],self.x)
+        self.higher = self.y
+        self.x.signalChange()
+        self.assertEqual(self.sorted[0],self.x)
+        self.assertEqual(self.sorted[1],self.y)
 
 class MapViewTestCase(DemocracyTestCase):
     def setUp(self):
@@ -325,8 +350,7 @@ class CallbackViewTestCase(DemocracyTestCase):
     def testChange(self):
         self.everything.addChangeCallback(self.call)
         self.x = database.DDBObject()
-        self.x.beginChange()
-        self.x.endChange()
+        self.x.signalChange()
         self.assertEqual(self.callcount,1)
     def testRemove(self):
         self.everything.addRemoveCallback(self.removeCall)
@@ -379,16 +403,24 @@ class CallbackViewTestCase(DemocracyTestCase):
 class SaveViewTestCase(DemocracyTestCase):
     def setUp(self):
         DemocracyTestCase.setUp(self)
+        self.tempdb = os.path.join(tempfile.gettempdir(), 'democracy-temp-db')
+        if not os.path.exists(self.tempdb):
+            os.makedirs(self.tempdb)
         self.everything = database.defaultDatabase
+        self.everything.liveStorage = storedatabase.LiveStorage(self.tempdb,
+                restore=False)
         self.x = database.DDBObject()
         self.y = database.DDBObject()
+    def tearDown(self):
+        shutil.rmtree(self.tempdb)
+        DemocracyTestCase.tearDown(self)
     def testSaveRestore(self):
-        storedatabase.saveDatabase(self.everything)
+        self.everything.liveStorage.saveDatabase()
         self.z = database.DDBObject()
         self.zz = database.DDBObject()
         last = self.zz.getID()
         self.x.remove()
-        storedatabase.restoreDatabase(self.everything)
+        self.everything.liveStorage = storedatabase.LiveStorage(self.tempdb)
         self.assertEqual(self.everything.len(),2)
         assert (self.everything[0].getID() == self.y.getID() or
                 self.everything[0].getID() == self.x.getID())
@@ -400,34 +432,15 @@ class SaveViewTestCase(DemocracyTestCase):
         assert database.DDBObject().getID() >= last
     def testLastID(self):
         last = self.y.getID()
-        storedatabase.saveDatabase(self.everything)
+        self.everything.liveStorage.saveDatabase()
         # Simulate restarting app
         self.y.remove()
         self.x.remove()
         database.DDBObject.lastID = 0 # This is implementation specific and
                              # needs to be updated when the
                              # implementation changes
-        storedatabase.restoreDatabase(self.everything)
-        assert database.DDBObject().getID() >= last
-    def testBackup(self):
-        storedatabase.saveDatabase(self.everything)
-        storedatabase.backedUp = False
-        storedatabase.saveDatabase(self.everything)
-        remove(config.get(prefs.DB_PATHNAME))
-        self.z = database.DDBObject()
-        self.zz = database.DDBObject()
-        last = self.zz.getID()
-        self.x.remove()
-        storedatabase.restoreDatabase(self.everything,config.get(prefs.DB_PATHNAME) + '.bak')
-        self.assertEqual(self.everything.len(),2)
-        assert (self.everything[0].getID() == self.y.getID() or
-                self.everything[0].getID() == self.x.getID())
-        if self.everything[0].getID() == self.y.getID():
-            self.assertEqual(self.everything[1].getID(),self.x.getID())
-        if self.everything[0].getID() == self.x.getID():
-            self.assertEqual(self.everything[1].getID(),self.y.getID())
-        self.assertEqual(self.everything[2],None)
-        assert database.DDBObject().getID() >= last
+        self.everything.liveStorage = storedatabase.LiveStorage(self.tempdb)
+        assert database.DDBObject().getID() > last
 
 class MapFilterRemoveViewTestCase(DemocracyTestCase):
     def setUp(self):
@@ -505,11 +518,9 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.mapped.addChangeCallback(self.call)
         mapped2.addChangeCallback(call2)
         if self.myfiltFunc(self.objlist[0]):
-            self.objlist[1].beginChange()
-            self.objlist[1].endChange()
+            self.objlist[1].signalChange()
         else:
-            self.objlist[0].beginChange()
-            self.objlist[0].endChange()
+            self.objlist[0].signalChange()
         self.assertEqual(self.callbacks,0)
         self.assertEqual(self.callbacks2,1)
         self.everything.recomputeFilters()
@@ -519,11 +530,9 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.assertEqual(self.callbacks,0)
         self.assertEqual(self.callbacks2,1)
         if self.myfiltFunc(self.objlist[0]):
-            self.objlist[0].beginChange()
-            self.objlist[0].endChange()
+            self.objlist[0].signalChange()
         else:
-            self.objlist[1].beginChange()
-            self.objlist[1].endChange()
+            self.objlist[1].signalChange()
         self.assertEqual(self.callbacks,1)
         self.assertEqual(self.callbacks2,2)
         self.everything.recomputeFilters()
@@ -554,11 +563,9 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.assertEqual(self.mapped.cur(),self.mapped[2])
         self.assertEqual(mapped2.cur(),mapped2[0])
         if self.myfiltFunc(self.objlist[0]):
-            self.objlist[1].beginChange()
-            self.objlist[1].endChange()
+            self.objlist[1].signalChange()
         else:
-            self.objlist[0].beginChange()
-            self.objlist[0].endChange()
+            self.objlist[0].signalChange()
         self.assertEqual(self.mapped.cur(),self.mapped[2])
         self.assertEqual(mapped2.cur(),mapped2[0])
         self.assertEqual(self.callbacks,0)
@@ -574,11 +581,9 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.assertEqual(self.callbacks,0)
         self.assertEqual(self.callbacks2,1)
         if self.myfiltFunc(self.objlist[0]):
-            self.objlist[0].beginChange()
-            self.objlist[0].endChange()
+            self.objlist[0].signalChange()
         else:
-            self.objlist[1].beginChange()
-            self.objlist[1].endChange()
+            self.objlist[1].signalChange()
         self.assertEqual(self.mapped.cur(),self.mapped[2])
         self.assertEqual(mapped2.cur(),mapped2[0])
         self.assertEqual(self.callbacks,1)
@@ -623,10 +628,8 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.assertEqual(mapped2.cur(),mapped2[0])
         self.assertEqual(self.callbacks,3)
         self.assertEqual(self.callbacks2,2)
-        self.objlist[10].beginChange()
-        self.objlist[10].endChange()
-        self.objlist[11].beginChange()
-        self.objlist[11].endChange()
+        self.objlist[10].signalChange()
+        self.objlist[11].signalChange()
         self.assertEqual(self.mapped.cur(),self.mapped[1])
         self.assertEqual(mapped2.cur(),mapped2[0])
         self.assertEqual(self.callbacks,4)
@@ -637,10 +640,8 @@ class FilterSortMapTestCase(DemocracyTestCase):
         self.myfiltFunc = lambda x:x is self.objlist[2]
         self.everything.recomputeFilters()
         self.assertEqual(self.callbacks2,3)
-        self.objlist[2].beginChange()
-        self.objlist[2].endChange()
-        self.objlist[3].beginChange()
-        self.objlist[3].endChange()
+        self.objlist[2].signalChange()
+        self.objlist[3].signalChange()
         self.assertEqual(self.callbacks2,4)
         
 class CursorTestCase(DemocracyTestCase):
@@ -705,8 +706,7 @@ class RecomputeMapTestCase(DemocracyTestCase):
 	self.everything.recomputeFilters()
 	self.everything.recomputeFilters()
 	temp = self.everything.getNext()
-	temp.beginChange()
-	temp.endChange()
+	temp.signalChange()
 	self.assertEqual(self.changeCalls,1)
 
 class FilterUpdateOnChange(DemocracyTestCase):
@@ -721,15 +721,13 @@ class FilterUpdateOnChange(DemocracyTestCase):
 	self.changeCalls = 0
     def testLoss(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[0].beginChange()
         self.origObjs[0].good = False
-        self.origObjs[0].endChange()
+        self.origObjs[0].signalChange()
         self.assertEqual(self.objs.len(),0)
     def testAdd(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[1].beginChange()
         self.origObjs[1].good = True
-        self.origObjs[1].endChange()
+        self.origObjs[1].signalChange()
         self.assertEqual(self.objs.len(),2)
 
 # Currently, we require that the database does NOT update maps on a change
@@ -749,15 +747,13 @@ class MapUpdateOnChange(DemocracyTestCase):
 	return temp
     def testLoss(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[0].beginChange()
         self.origObjs[0].good = False
-        self.origObjs[0].endChange()
+        self.origObjs[0].signalChange()
         self.assertEqual(self.objs.len(),1)
     def testAdd(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[1].beginChange()
         self.origObjs[1].good = True
-        self.origObjs[1].endChange()
+        self.origObjs[1].signalChange()
         self.assertEqual(self.objs.len(),1)
 
 class SortUpdateOnChange(DemocracyTestCase):
@@ -772,15 +768,13 @@ class SortUpdateOnChange(DemocracyTestCase):
 	self.changeCalls = 0
     def testLoss(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[0].beginChange()
         self.origObjs[0].good = False
-        self.origObjs[0].endChange()
+        self.origObjs[0].signalChange()
         self.assertEqual(self.objs.len(),0)
     def testAdd(self):
         self.assertEqual(self.objs.len(),1)
-        self.origObjs[1].beginChange()
         self.origObjs[1].good = True
-        self.origObjs[1].endChange()
+        self.origObjs[1].signalChange()
         self.assertEqual(self.objs.len(),2)
 
 class IDBaseTraversal(DemocracyTestCase):
@@ -841,18 +835,30 @@ class IDBaseTraversal(DemocracyTestCase):
 #         self.remove100()
 #         thread.join()
 
-class IndexFilterTest(DemocracyTestCase):
+class IndexFilterTestBase(DemocracyTestCase):
     def setUp(self):
         DemocracyTestCase.setUp(self)
         self.everything = database.defaultDatabase
         self.addCallbacks = 0
         self.removeCallbacks = 0
         self.changeCallbacks = 0
+    def addCallback(self,value,id):
+        self.addCallbacks += 1
+    def removeCallback(self,value,id):
+        self.removeCallbacks += 1
+    def changeCallback(self,value,id):
+        self.changeCallbacks += 1
+
+class IndexFilterTest(IndexFilterTestBase):
+    def setUp(self):
+        IndexFilterTestBase.setUp(self)
         self.shift = 0
     def mod10(self, x):
         return (x.getID() + self.shift) % 10
     def mod100(self, x):
         return (x.getID() + self.shift) % 100
+    def sortIndexFunc(self, x, y):
+        return cmp(x.myValue, y.myValue)
     def sortFunc(self, x, y):
         x = x.getID()
         y = y.getID()
@@ -862,19 +868,22 @@ class IndexFilterTest(DemocracyTestCase):
             return 1
         else:
             return 0
-    def addCallback(self,value,id):
-        self.addCallbacks += 1
-    def removeCallback(self,value,id):
-        self.removeCallbacks += 1
-    def changeCallback(self,value,id):
-        self.changeCallbacks += 1
     def testBasicIndexFilter(self):
         for x in range(0,100):
             database.DDBObject()
         self.everything.createIndex(self.mod10)
+        filtered = self.everything.filterWithIndex(self.mod10,0)
+        self.assertEqual(filtered.len(),10)
         for x in range(0,50):
             database.DDBObject()
-        filtered = self.everything.filterWithIndex(self.mod10,0)
+        self.assertEqual(filtered.len(),15)
+        for i in range(10):
+            obj = self.everything.getItemWithIndex(self.mod10, i)
+            self.assertEqual(self.mod10(obj), i)
+        obj = self.everything.getItemWithIndex(self.mod10, 10)
+        self.assertEqual(obj, None)
+        obj = self.everything.getItemWithIndex(self.mod10, -1, default=123123)
+        self.assertEqual(obj, 123123)
         filtered.addAddCallback(self.addCallback)
         filtered.addRemoveCallback(self.removeCallback)
         filtered.addChangeCallback(self.changeCallback)
@@ -887,8 +896,7 @@ class IndexFilterTest(DemocracyTestCase):
         filtered[0].remove()
         self.assertEqual(filtered.len(),19)
         self.assertEqual(self.removeCallbacks,1)
-        filtered[0].beginChange()
-        filtered[0].endChange()
+        filtered[0].signalChange()
         self.assertEqual(self.changeCallbacks,1)
 
         obj = filtered[0]
@@ -896,11 +904,48 @@ class IndexFilterTest(DemocracyTestCase):
         for x in range(0,50):
             database.DDBObject()
         self.assertEqual(self.addCallbacks,5)
-        obj.beginChange()
-        obj.endChange()
+        obj.signalChange()
         self.assertEqual(self.changeCallbacks,1)
         obj.remove()
         self.assertEqual(self.removeCallbacks,1)
+    def testIndexChanges(self):
+        class IndexedObject(database.DDBObject):
+            def __init__(self, myValue):
+                database.DDBObject.__init__(self)
+                self.myValue = myValue
+        def indexFunc(obj):
+            return obj.myValue
+        foo = IndexedObject('blue')
+        bar = IndexedObject('red')
+        baz = IndexedObject('red')
+        self.everything.createIndex(indexFunc)
+        blueView = self.everything.filterWithIndex(indexFunc, 'blue')
+        redView = self.everything.filterWithIndex(indexFunc, 'red')
+        self.assertEquals(blueView.len(), 1)
+        self.assertEquals(redView.len(), 2)
+        baz.myValue = 'blue'
+        baz.signalChange()
+        self.assertEquals(blueView.len(), 2)
+        self.assertEquals(redView.len(), 1)
+        # test changing to a new view that we've never referenced before.
+        foo.myValue = 'green'
+        foo.signalChange()
+        greenView = self.everything.filterWithIndex(indexFunc, 'green')
+        self.assertEquals(blueView.len(), 1)
+        self.assertEquals(redView.len(), 1)
+        self.assertEquals(greenView.len(), 1)
+    def testRemoveIndexedView(self):
+        for x in range(0,100):
+            database.DDBObject()
+        self.everything.createIndex(self.mod10)
+        views = [self.everything.filterWithIndex(self.mod10, i) \
+                for i in range(10)]
+        # remove half the views with parent.removeView()
+        for view in views[:5]:
+            self.everything.removeView(view)
+        # remove the other half with unlink()
+        for view in views[5:]:
+            view.unlink()
     def testRecomputeIndex(self):
         for x in range(0,100):
             database.DDBObject()
@@ -948,10 +993,290 @@ class IndexFilterTest(DemocracyTestCase):
         assert ( (end-mid) - (mid-start) < (mid-start)/2)
         filtered = self.everything.filterWithIndex(self.mod100,0).sort(self.sortFunc)
         self.assertEqual(filtered.len(),100)
+    def testSortedIndex(self):
+        class IndexedObject(database.DDBObject):
+            def __init__(self, myValue):
+                self.myValue = myValue
+                database.DDBObject.__init__(self)
+        self.everything.createIndex(self.mod10,self.sortIndexFunc)
+        self.objects = []
+        for x in range(100):
+            self.objects.append(IndexedObject(x))
+
+        # Test basic sorting
+        filtered = self.everything.filterWithIndex(self.mod10,0)
+        self.assertEqual(filtered.len(), 10)
+        last = None
+        for obj in filtered:
+            if last is not None:
+                self.assert_(last.myValue < obj.myValue)
+            last = obj
+
+        # Test changing values without signaling
+        filtered[0].myValue = 1000
+        filtered[1].myValue = -1000
+
+        unordered = False
+        last = None
+        filtered.resetCursor()
+        for obj in filtered:
+            if last is not None:
+                if not (last.myValue < obj.myValue):
+                    unordered = True
+            last = obj
+        self.assert_(unordered)
+
+        # Test that things get re-ordered correctly on signalChange
+        filtered[0].signalChange()
+        filtered[1].signalChange()
+        last = None
+        filtered.resetCursor()
+        for obj in filtered:
+            if last is not None:
+                self.assert_(last.myValue < obj.myValue)
+            last = obj
+        # Test that a new filter on the index is sorted
+        filtered2 = self.everything.filterWithIndex(self.mod10,0)
+        self.assertEqual(filtered2.len(), 10)
+        last = None
+        for obj in filtered2:
+            if last is not None:
+                self.assert_(last.myValue < obj.myValue)
+            last = obj
+
+class MultiIndexed(database.DDBObject):
+    def __init__(self, indexValues):
+        self.indexValues = indexValues
+        database.DDBObject.__init__(self)
+def testMultiIndex(obj):
+    return obj.indexValues
+
+class MultiIndexTestCase(IndexFilterTestBase):
+    def setUp(self):
+        IndexFilterTestBase.setUp(self)
+        random.seed(12341234)
+        self.allObjects = []
+        self.objectsByValueCount = {}
+        for i in range(20):
+            self.newObject()
+        self.everything.createIndex(testMultiIndex, multiValued=True)
+
+    def genRandomValues(self):
+        values = set()
+        for i in xrange(random.randint(0, 4)):
+            values.add(random.randint(0, 10))
+        return list(values)
+
+    def newObject(self):
+        indexValues = self.genRandomValues()
+        obj = MultiIndexed(indexValues)
+        self.allObjects.append(obj)
+        try:
+            self.objectsByValueCount[len(indexValues)].append(obj)
+        except KeyError:
+            self.objectsByValueCount[len(indexValues)] = [obj]
+
+    def checkViews(self):
+        viewsShouldHave = {}
+        for obj in self.allObjects:
+            for value in obj.indexValues:
+                try:
+                    viewsShouldHave[value].add(obj)
+                except KeyError:
+                    viewsShouldHave[value] = set([obj])
+        for value, goal in viewsShouldHave.items():
+            filtered = self.everything.filterWithIndex(testMultiIndex, value)
+            reality = set([obj for obj in filtered])
+            self.assertEqual(goal, reality)
+
+    def testInitalViews(self):
+        self.checkViews()
+
+    def testRemove(self):
+        while self.allObjects:
+            obj = self.allObjects.pop()
+            obj.remove()
+            self.checkViews()
+
+    def testChange(self):
+        for obj in self.allObjects:
+            obj.indexValues = self.genRandomValues()
+            obj.signalChange(needsSave=False)
+            self.checkViews()
+
+    def testCallbacks(self):
+        filtered = self.everything.filterWithIndex(testMultiIndex, 0)
+        filtered.addAddCallback(self.addCallback)
+        filtered.addRemoveCallback(self.removeCallback)
+        filtered.addChangeCallback(self.changeCallback)
+        addCallbackGoal = removeCallbackGoal = changeCallbackGoal = 0
+        for obj in self.allObjects:
+            newValues = self.genRandomValues()
+            if 0 in newValues:
+                if 0 not in obj.indexValues:
+                    addCallbackGoal += 1
+                else:
+                    changeCallbackGoal += 1
+            elif 0 in obj.indexValues:
+                removeCallbackGoal += 1
+            obj.indexValues = newValues
+            obj.signalChange(needsSave=False)
+            self.assertEquals(self.changeCallbacks, changeCallbackGoal)
+            self.assertEquals(self.addCallbacks, addCallbackGoal)
+            self.assertEquals(self.removeCallbacks, removeCallbackGoal)
+
+class ReSortTestCase(DemocracyTestCase):
+    def setUp(self):
+        DemocracyTestCase.setUp(self)
+        self.everything = database.defaultDatabase
+        self.addCallbacks = 0
+        self.removeCallbacks = 0
+        self.changeCallbacks = 0
+        self.objlist = []
+        for x in range(0,10):
+            self.objlist.append(SortableObject(x))
+        self.sorted = self.everything.sort(self.sortFunc, resort = True)
+        self.sorted.addAddCallback(self.addCall)
+        self.sorted.addRemoveCallback(self.removeCall)
+        self.sorted.addChangeCallback(self.changeCall)
+
+    def sortFunc(self, x, y):
+        return cmp(x.value,y.value)
+
+    def addCall(self, obj, id):
+        # We have a convention of setting the cursor after the object
+        self.assertEqual(self.sorted.getPrev().getID(),id)
+        
+        self.addCallbacks += 1
+
+    def removeCall(self, obj, id):
+        self.removeCallbacks += 1
+
+    def changeCall(self, obj, id):
+        self.changeCallbacks += 1
+
+    def testResort(self):
+        self.sorted.resetCursor()
+        last = None
+        for obj in self.sorted:
+            if last is not None:
+                self.assert_(last.value < obj.value)
+            last = obj
+            
+        self.objlist[0].value = 100
+        self.objlist[0].signalChange()
+
+        self.sorted.resetCursor()
+        last = None
+        for obj in self.sorted:
+            if last is not None:
+                self.assert_(last.value < obj.value)
+            last = obj
+        self.assertEqual(self.addCallbacks, 1)
+        self.assertEqual(self.removeCallbacks, 1)
+        self.assertEqual(self.changeCallbacks, 0)
+
+class SortingFilterTestCase(DemocracyTestCase):
+    def setUp(self):
+        DemocracyTestCase.setUp(self)
+        self.everything = database.defaultDatabase
+        self.sortCalls = 0
+        self.objs = []
+
+    def sortFunc(self, x, y):
+        self.sortCalls += 1
+        return cmp(x.value,y.value)
+
+    def testSort(self):
+        sortView = self.everything.sort(self.sortFunc)
+        for x in range(2000):
+            a = SortableObject(2000)
+            self.objs.append(a)
+        initialSorts = self.sortCalls
+        self.sortCalls = 0
+        filtView = sortView.filter(lambda x:True,sortFunc=self.sortFunc)
+        filterSorts = self.sortCalls
+        self.sortCalls = 0
+
+        self.assertEqual(sortView.len(),filtView.len())
+        sortView.resetCursor()
+        filtView.resetCursor()
+        for obj in sortView:
+            self.assertEqual(obj,filtView.getNext())
+
+        self.objs[-1].value = -10
+        self.objs[-1].signalChange()
+        self.objs[-2].value = 0
+        self.objs[-2].signalChange()
+        self.assertEqual(self.sortCalls, 0)
+        sortView.unlink()
+        filtView.unlink()
+
+    def testResort(self):
+        sortView = self.everything.sort(self.sortFunc, resort = True)
+        for x in range(2000):
+            a = SortableObject(x)
+            self.objs.append(a)
+        initialSorts = self.sortCalls
+        self.sortCalls = 0
+        filtView = sortView.filter(lambda x:True,sortFunc=self.sortFunc,
+                                   resort=True)
+        filterSorts = self.sortCalls
+        self.sortCalls = 0
+
+        self.assertEqual(sortView.len(),filtView.len())
+        sortView.resetCursor()
+        filtView.resetCursor()
+        last = None
+        for obj in sortView:
+            self.assertEqual(obj,filtView.getNext())
+            if last != None:
+                self.assert_(obj.value >= last.value)
+            last = obj
+
+        self.objs[-1].value = -10
+        self.objs[-1].signalChange()
+        self.objs[-2].value = -1
+        self.objs[-2].signalChange()
+        self.assert_(self.sortCalls > 0)
+
+        sortView.resetCursor()
+        filtView.resetCursor()
+        last = None
+        for obj in sortView:
+            self.assertEqual(obj,filtView.getNext())
+            if last != None:
+                self.assert_(obj.value >= last.value)
+            last = obj
+        
+        sortView.unlink()
+        filtView.unlink()
+
+    def testPerformance(self):
+        # Filtering an already sorted list with a sort must be O(n)
+        #
+        # In other words, we need to make sure that the list isn't
+        # being resorted
+        sortView = self.everything.sort(self.sortFunc)
+        initialSorts = []
+        filterSorts = []
+        for n in [100, 900, 9100]:
+            for x in range(n):
+                a = SortableObject(n)
+                self.objs.append(a)
+            initialSorts.append(self.sortCalls)
+            self.sortCalls = 0
+            filtView = sortView.filter(lambda x:True,sortFunc=self.sortFunc)
+            filterSorts.append(self.sortCalls)
+            self.sortCalls = 0
+        ratio1 = float(filterSorts[1])/filterSorts[0]
+        ratio2 = float(filterSorts[2])/filterSorts[1]
+
+        # Make sure the ratios are within 1% of each other
+        self.assert_(abs(ratio1-ratio2)/ratio1 < 0.01)
 
 
 #FIXME: Add test for explicitly recomputing sorts
-#FIXME: Add test for recomputing sorts on endChange()
 #FIXME: Add test for unlink()
 
 if __name__ == "__main__":
